/** 
 * Parse a 'var' or 'const' statement, or a 'var' init list in a for statement.
 * @param inFor true if we are currently in the midst of the initclause of a for.
 * @param inStatement true if called in a statement (as opposed to anexpression) context
 * @param declType A token value: either VAR, CONST, or LET depending oncontext.
 * @return The parsed statement
 * @throws IOException
 * @throws ParserException
 */
private Node variables(boolean inFor,boolean inStatement,int declType) throws IOException, ParserException {
  Node result=nf.createVariables(inStatement ? declType : Token.COMMA,ts.getLineno());
  Node destructuringInit=null;
  boolean first=true;
  for (; ; ) {
    Node destructuring=null;
    String s=null;
    int tt=peekToken();
    if (tt == Token.LB || tt == Token.LC) {
      destructuring=primaryExpr();
    }
 else {
      mustMatchToken(Token.NAME,"msg.bad.var");
      s=ts.getString();
      if (!first)       decompiler.addToken(Token.COMMA);
      first=false;
      decompiler.addName(s);
      defineSymbol(declType,s);
    }
    Node init=null;
    if (matchToken(Token.ASSIGN)) {
      decompiler.addToken(Token.ASSIGN);
      init=assignExpr(inFor);
    }
    if (destructuring != null) {
      if (init == null) {
        if (!inFor)         reportError("msg.destruct.assign.no.init");
        nf.addChildToBack(result,destructuring);
      }
 else {
        if (destructuringInit == null) {
          destructuringInit=new Node(Token.COMMA);
        }
        nf.addChildToBack(result,nf.createDestructuringAssignment(declType,destructuring,init));
      }
    }
 else {
      if (inStatement) {
        Node name=nf.createName(s);
        if (init != null)         nf.addChildToBack(name,init);
        nf.addChildToBack(result,name);
      }
 else       if (init != null) {
        Node string=nf.createString(s);
        string.setScope(currentScope);
        nf.addChildToBack(result,nf.createBinary(Token.SETVAR,string,init));
      }
    }
    if (!matchToken(Token.COMMA))     break;
  }
  return result;
}
