package mining.commits;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class GitLogger {
	
	public static List<Commit> loadCommits(String path){

		List<Commit> commits = new ArrayList<Commit>();
		
		List<String> lines = Helper.readFile(path);
		
		if (lines == null)
			return null;
		
		System.out.println(lines.size());
		
		Commit m = new Commit();
		String subject = "";
		String body = "";
		String visiting = "";
		List<CFile> cFiles = new ArrayList<CFile>();
		
		for (String line : lines){
			if (line.startsWith("Hash") && Helper.matchCommitHash(line)){
				m.setcFiles(cFiles);
				m.setSubject(subject);
				m.setBody(body);
				commits.add(m);
				
				m = new Commit();
				subject = "";
				body = "";
				cFiles = new ArrayList<CFile>();
				
				m.setHASH(line.split(":")[1]);
				//System.out.println(line);
				
			} else if (line.startsWith("Author Name:")){
				m.setAuthorName(line.substring("Author Name:".length()).trim());
			} else if (line.startsWith("Author Date:")){
				m.setAuthorDate(line.substring("Author Date:".length()).trim());
			} else if (line.startsWith("Committer Name:")) {
				m.setCommitterName(line.substring("Committer Name:".length()).trim());
			} else if (line.startsWith("Committer Date:")) {
				m.setCommitterDate(line.substring("Committer Date:".length()).trim());
			} else if (line.startsWith("Subject:")) {
				subject += line.substring("Subject:".length()).trim();
				visiting = "Subject";
			} else if (line.startsWith("Body:")) {
				body += line.substring("Body:".length()).trim();
				visiting = "Body";
			} else if (line.startsWith("Committed Files:")) {
				visiting = "files";
			} else if (!line.isEmpty() && visiting.equalsIgnoreCase("subject")) {
				subject += line.trim() + "\n";
			} else if (!line.isEmpty() && visiting.equalsIgnoreCase("body")) {
				body += line.trim() + "\n";
			}  else if (!line.isEmpty() && visiting.equalsIgnoreCase("files")) {
				cFiles.add(new CFile(line.trim()));
			} 
		}	
		
		m.setcFiles(cFiles);
		m.setSubject(subject);
		m.setBody(body);
		
		commits.add(m);
		commits.remove(0);
		
		return commits;
	}

	public static List<Commit> loadName_Status_Commits(String path) {
List<Commit> commits = new ArrayList<Commit>();
		
		List<String> lines = Helper.readFile(path);
		
		if (lines == null)
			return null;
		
		Commit m = new Commit();
		String subject = "";
		
		List<CFile> cFiles = new ArrayList<CFile>();
		
		for (String line : lines){
			if (line.startsWith("commit") && Helper.matchCommitHash(line)){
				m.setcFiles(cFiles);
				m.setSubject(subject);
				commits.add(m);
				
				m = new Commit();
				subject = "";
				cFiles = new ArrayList<CFile>();
				
				m.setHASH(line.split(" ")[1]);
				
				
			} else if (line.startsWith("Author:")){
				m.setAuthorName(line.substring("Author:".length()).trim());
			} else if (line.startsWith("Date:")){
				m.setAuthorDate(line.substring("Date:".length()).trim());
			} else if (line.startsWith(" ") && !line.isEmpty()) {
				subject += line;
			} else if (!line.isEmpty()){
				cFiles.add(new CFile(line.trim()));
			} 
		}	
		
		m.setcFiles(cFiles);
		m.setSubject(subject);
		
		
		commits.add(m);
		commits.remove(0);
		
		return commits;
		
	}
	
	
	public static List<Commit> getTestCommits(List<Commit> commits) {
		
		List<Commit> testCommits = new ArrayList<Commit>();
		
		for (Commit c : commits) {
			List<CFile> cFiles = c.getcFiles();
			for (CFile f : cFiles) {
				if (f.isTestFile()) {
					testCommits.add(c);
					break;
				}
			}
		}
		
		return testCommits;
	}
	
	public static List<Commit> getPureTestCommits(List<Commit> commits){
		List<Commit> pureTestCommits = new ArrayList<Commit>();
		for (Commit c : commits) {
			if (c.isPureTestCommit()) {
				pureTestCommits.add(c);
			}
		}
		return pureTestCommits;
	}
	
	public static List<Commit> getPureCUTCommit(List<Commit> commits) {
		List<Commit> pureCUTCommits = new ArrayList<Commit>();
		for (Commit c : commits) {
			if (c.isPureCUTCommit()) {
				pureCUTCommits.add(c);
			}
		}
		return pureCUTCommits;
	}
	
	public static int getNoAddFilesFromCommits (List<Commit> commits) {
		int total = 0;
		for (Commit c : commits) {
			total += c.getNoAddFiles();
		}
		return total;
	}
	
	public static int getNoModifiedFilesFromCommits (List<Commit> commits) {
		int total = 0;
		for (Commit c : commits) {
			total += c.getNoModifiedFiles();
		}
		return total;
	}
	
	public static int getNoCommittedFiles (List<Commit> commits) {
		int total = 0;
		for (Commit c : commits) {
			total += c.getTotalFiles();
		}
		return total;
	}
	
	public static int getNoNonAddandModifiedFiles (List<Commit> commits) {
		return (getNoCommittedFiles(commits) - getNoAddFilesFromCommits(commits) - getNoModifiedFilesFromCommits(commits));
	}
	
	
	public static void matchCommits(List<Commit> commits, String matchingPattern){
		int count = 0;
		for (Commit c : commits) {
			if ((c.getSubject() != null) && (Helper.matchPattern(c.getSubject(), matchingPattern))){
				count++;
				//System.out.println(count + ": "+ c.getSubject());
				c.print();
			}
		}
		//System.out.println("Number of commits related to matching pattern: " + matchingPattern + ": ");
		System.out.println(count);
	}
	
	public static List<Commit> matchCommits2(List<Commit> commits, String matchingPattern){
		int count = 0;
		List<Commit> results = new ArrayList<Commit>();
		for (Commit c : commits) {
			if ((c.getSubject() != null) && (Helper.matchPattern(c.getSubject(), matchingPattern))){
				count++;
				//System.out.println(count + ": "+ c.getSubject());
				results.add(c);
				//c.print();
			}
		}
		//System.out.println("Number of commits related to matching pattern: " + matchingPattern + ": ");
		System.out.println(count);
		return results;
	}
	
	public static void findAddCommits(){
		String addCommitsReg = Helper.createRegToFilterCommits(Helper.COMMIT_TYPE.ADD);
		matchCommits(loadCommits(Helper.LOG_FILE_PATH), addCommitsReg);
	}
	
	public static void findModifyCommits(){
		String modifyCommitsReg = Helper.createRegToFilterCommits(Helper.COMMIT_TYPE.MODIFICATION);
		matchCommits(loadCommits(Helper.LOG_FILE_PATH), modifyCommitsReg);
	}
	
	public static void findTestcaseCommits(){
		String testReg = Helper.createRegToFilterCommits(Helper.COMMIT_TYPE.TESTCASE);
		matchCommits(loadCommits(Helper.LOG_FILE_PATH), testReg);
	}
	
	public static void printResult_Test_Failure_Fix_Commits(String filePath){
		List<Commit> commits = loadName_Status_Commits(filePath);
		System.out.println("All commmits: " + commits.size());
		
		List<Commit> pureTestCommits = getPureTestCommits(commits);
		System.out.println("Pure Test Commits: " + pureTestCommits.size());
		System.out.println("Test Files Committed: " + Helper.getNumberofCommittedFiles(pureTestCommits));
		
		List<Commit> pureCUTCommits = getPureCUTCommit(commits);
		System.out.println("Pure CUT Commits: " + pureCUTCommits.size());
		System.out.println("CUT Files Committed: " + Helper.getNumberofCommittedFiles(pureCUTCommits));
		
		System.out.println("Mixed Test and CUT commits: " + (commits.size() - pureTestCommits.size() - pureCUTCommits.size()));
		System.out.println("Test Files Committed: " + (Helper.getNumberofTestCommittedFiles(commits) - Helper.getNumberofCommittedFiles(pureTestCommits)));
		System.out.println("CUT Files Committed: " + (Helper.getNumberofCUTCommitedFiles(commits) - Helper.getNumberofCommittedFiles(pureCUTCommits)));
		
	}
	
	
	public static void extractShortHashes (String filePath) {
		List<String> hashes = Helper.extractShortHashes(filePath);
		System.out.println(Helper.straightenList(hashes));
		
	}
	
	public static String extractHashList(List<Commit> commits){
		String s = "";
		for (Commit c : commits){
			s += c.getHASH().trim() + "\n";
		}
		return s.trim();
	}	
	
	/**
	 * loop through raw log files (each corresponding to a project)
	 * and extract commit hash that satisfies some matching patters (i.e., contain: fix + test failure)
	 * and save results to two files: a full log file, and hash only file
	 * @param inputFolder
	 * @param outputFolder
	 */
	public static int extractMatchHashes(String inputFolder, String outputFolder) throws IOException{
		File[] logFiles = new File(inputFolder).listFiles();
		
		int total = 0;
		
		for (File file : logFiles){
			if (file.getName().endsWith(".txt")){
				String inputFileName = file.getName().substring(0, file.getName().length()-4);
				String.format("Start processing on project: %s", inputFileName);
				String outputCommitFile = outputFolder + "/" + inputFileName + "_commits" + ".txt";
				String outputHashFile = outputFolder + "/" + inputFileName + ".txt";
				total += loadFilterAndSaveCommmits(inputFileName, file.getAbsolutePath(), outputCommitFile, outputHashFile);
				String.format("Read %s ---> write %s ---> write %s", file.getName(), outputCommitFile, outputHashFile);
			}
		}
		
		System.out.print("Total matched hashes found: " + total);
		
		return total;

	}
	private static int loadFilterAndSaveCommmits(String projectName, String inputFile, String outputCommitFile, String outputHashFile) throws IOException{
		List<Commit> commits = loadCommits(inputFile);
		commits = matchCommits2(commits, Helper.FIX_TESTFAILURE);
		commits = matchCommits2(commits, "(" + projectName + ")"+ Helper.BUG_TRACKING_NUMBER);
		//Helper.writeCommitsToFile(outputCommitFile, commits);
		String hashList = extractHashList(commits);
		Helper.writeStringtoFile(outputHashFile, hashList);
		return commits.size();
	}
	
	public static void main(String[] args) throws IOException{
		
//		String path = "/Users/assist/dataset/TCM/resources/commit-logs/mahout_test-failure_fix.txt";
//		extractShortHashes(path);
//		
//		String nameStatusPath = "/Users/assist/dataset/TCM/resources/commit-logs/mahout_test-failure_fix_name-status.txt";
//		printResult_Test_Failure_Fix_Commits(nameStatusPath);
		
//		List<Commit> testCommits = getTestCommits(commits);
//		System.out.println("Commits related to tests: " + testCommits.size());
//		
//		List<Commit> pureTestCommits = getPureTestCommits(testCommits);
//		
//		System.out.println("Pure test commits: " + pureTestCommits.size());
//		System.out.println("Mixed test and implementation commits: " + (testCommits.size() - pureTestCommits.size()));
//		
//		System.out.println("Commits not related to tests: " + (commits.size()-testCommits.size()));
//		
//		System.out.println("How many files commited among Pure-Test commits: " + getNoCommittedFiles(pureTestCommits));
//		System.out.println("How many files are added: " + getNoAddFilesFromCommits(pureTestCommits));
//		System.out.println("How many files are modified: " + getNoModifiedFilesFromCommits(pureTestCommits));
//		System.out.println("How many files are neither added and modified: " + getNoNonAddandModifiedFiles(pureTestCommits));
		
		
		
		
		
//		for (Commit c : testCommits) {
//			c.print();
//		}
		
//		System.out.println("Listing all commits related to test cases");
//		findTestcaseCommits();
//		
//		System.out.println("-------fixing test cases -----------");
//		findModifyCommits();
		
		
//		System.out.print("Number of commits related to test cases: ");
//		matchCommits(commits, Helper.Tests_REG);
//		System.out.println();
		
//		System.out.println("----------Finding commits related to adding test cases--------");
//		matchCommits(commits,Helper.ADD_REG);
//		System.out.println();
		
//		List<Commit> commits = loadCommits(Helper.LOG_FILE_PATH);
//		System.out.println("Total of commits: " + commits.size());
//		
//		System.out.println("----------Finding commits related to fixing test cases--------");
//		
//		List<Commit> cList = matchCommits2(matchCommits2(commits,Helper.FIX_TESTFAILURE), Helper.BUG_TRACKING_NUMBER);
//		
//		Helper.writeCommitsToFile("/Users/assist/dataset/TCM/resources/commit-logs/aries_found.txt", cList);
//		
//		String hashes = extractHashList(cList);
//		
//		Helper.writeStringtoFile("/Users/assist/dataset/TCM/resources/commit-logs/aries_found_hash.txt", hashes);
//		
//		System.out.println();
		
		
//		System.out.println("----------Finding commits related to modifying test cases--------");
//		matchCommits(commits,Helper.MODIFY_REG);
//		System.out.println();
		
		extractMatchHashes(Helper.LOG_INPUT_FOLDER, Helper.HASH_OUTPUT_FOLDER);
		
		
	}
	
}
