package rtp.extractor;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.ASTParser;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.Document;

import rtp.utility.FileSystem;

public class ClassExtractor {
	private static Document fDocument;
	
	/*
	 * Added by Tung 
	 * To resolve the issue of Type Safe
	 */
	public static <T> List<T> castList(Class<? extends T> clazz, Collection<?> c) {
	    List<T> r = new ArrayList<T>(c.size());
	    for(Object o: c)
	      r.add(clazz.cast(o));
	    return r;
	}
	

	public static void main(String args[]) throws CoreException, IOException{
		System.out.println("Invoking comment extractor mehod");

		FileSystem fs = new FileSystem();
		String version = "1.0";
		//String codeDirectory = "/Users/ripon/RTS/subjects-ripon/commons-lang3.03/src/test/java3.03";
		//String codeDirectory = "/Users/ripon/RTS/subjects-ripon/jaxen1.1b9/src/java/test1.9";
		//String codeDirectory = "/Users/ripon/RTS/subjects-ripon/joda-time"+version+"/src/test"+version;
		//String codeDirectory = "/Users/ripon/RTS/subjects-ripon/timeandmoney4.0/tests/src4.0";
		//String codeDirectory = "/Users/ripon/RTS/subjects-ripon/jaxen"+version+"/src/test"+version;
		//String codeDirectory = "/Users/ripon/RTS/subjects-ripon/mime4j0.61/src/test/java0.60";
		//String codeDirectory = "/Users/ripon/RTS/subjects-ripon/xstream"+version+"/src/test"+version;
		//		String codeDirectory = "/Users/ripon/RTS/subjects-ripon/Ant0.0/src/testcases0";
		//		String outputDir = "/Users/ripon/RTS/result/Ant0.0/class-doc";

		//String codeDirectory = "/Users/ripon/aspectj/trunk/org.aspectj";

//		String codeDirectory = "/Users/assist/rtp/dataset/subject_data/apache-ant-1.9.5/src/tests";
//		String outputDir = "/Users/assist/rtp/dataset/subject_data/apache-ant-1.9.5/class-doc";
		
		String codeDirectory = "/Users/assist/dataset/cassandra";
		String outputDir = "/Users/assist/dataset/rtp/dataset/cassandra/class-flatten";



		int offset = codeDirectory.length();
		List<String> files = fs.getAllJavaFiles(codeDirectory, codeDirectory.length());
		Iterator <String> it = files.iterator();

		int count = 1;
		int fileCount = 0;
		
		System.out.println("Processing");
		
		while(it.hasNext()){

			fileCount++;

			String filePath = it.next();
			System.out.println(fileCount+"\t"+filePath);
			
			
			//		if(count<4824){
			//			count++;
			//			continue;
			//		}





			String classFQN = filePath.substring(offset+1).replaceAll("/","."); // fully qualified name of a class 
			String part[]= classFQN.split("\\.");
			String className = part[part.length-2];


			// ASTParser parser = ASTParser.newParser(AST.JLS3); //comment out by Tung
			ASTParser parser = ASTParser.newParser(AST.JLS8); //JLS3 is deprecated by Tung


			String source = FileSystem.readFile(filePath);

			//System.out.println(source);

			parser.setSource(source.toCharArray());
			parser.setKind(ASTParser.K_COMPILATION_UNIT); // source to be parsed as a compilation unit
			final CompilationUnit cu = (CompilationUnit) parser.createAST(null); //create AST of a class file 


			//List<ASTNode> list = cu.getCommentList(); //comment out by Tung
			List<ASTNode> list = castList(ASTNode.class, cu.getCommentList()); // get all AST comment nodes in the class
			
			String key = "";

			String fileName = "/class-"+count;

			File file = new File(outputDir+fileName);

			// if file doesnt exists, then create it
			if (!file.exists()) {
				file.createNewFile();
			}

			FileWriter fw = new FileWriter(file.getAbsoluteFile());
			BufferedWriter bw = new BufferedWriter(fw);


			key = classFQN.substring(0, classFQN.length()-5);

			bw.write("<DOC>\n\t<DOCNO>"+key+" </DOCNO>\n\t<TEXT>");
			bw.newLine();

			bw.write(className);
			bw.newLine();

			bw.write(PreProcessor.transform(className));
			bw.newLine();

			fDocument = new Document(source);





			for(int k=0;k<list.size();k++){

				//System.out.println("COMMENT:\n\n");
				//System.out.println(list1.size());
				//System.out.println(list);
				ASTNode cComment = list.get(k);
				String comStr = getCommentString(cComment);
				//System.out.println(comStr);
				bw.write(PreProcessor.process(comStr));
				bw.newLine();
			}	
			//bw.write("code");
			//bw.newLine();
			GenericVisitor visitor = new GenericVisitor(bw);

			try{
				cu.accept(visitor);
			}
			catch(Exception e){
				e.printStackTrace();
			}
			bw.write("\t</TEXT>\n</DOC>");
			bw.newLine();
			bw.close();
			count++;
		}    

		System.out.println("End");
	}

	private static String getCommentString(ASTNode node) {
		try {
			return fDocument.get(node.getStartPosition(), node.getLength());
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
		return null;
	}


}

