package rtp.extractor;

import java.io.BufferedWriter;
import static java.nio.file.StandardCopyOption.*;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Document;
import org.apache.commons.io.FileUtils;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTParser;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import rtp.utility.FileSystem;
import td.utils.FileHandler;

public class MethodExtractor4BL {
	
	private static Document fDocument;
//	
//	private static String getCommentString(ASTNode node) {
//		try {
//			return fDocument.get(node.getStartPosition(), node.getLength());
//		} catch (BadLocationException e) {
//			e.printStackTrace();
//		}
//		return null;
//	}
	
	public static void extractJavaFilesForBugLocator(String inputProjectDir, String outputFilePath) throws IOException {
		FileSystem fs = new FileSystem();
		int offset = inputProjectDir.length();
		List<String> files = fs.getAllJavaFiles(inputProjectDir, offset);
		
		List<String> FQNfiles = new ArrayList<String>();
		
		for (String file : files) {
			String FQNfile = "<file>" + file.split("pre-fix\\/")[1].replaceAll("/", ".").trim() + "</file>";
			FQNfiles.add(FQNfile);
		}
		
		FileHandler.writeListToFile(FQNfiles, outputFilePath);
	}
	
	public static void extractingDataForBugLocator(String inputAllDir, String outputAllDir) throws IOException {
		
		File[] projects = new File(inputAllDir).listFiles();
		for (File project : projects) {
			if (project.getName().startsWith(".")){
				continue;
			}
			
			File[] versions = new File(project.getAbsolutePath()).listFiles();
			for(File version : versions) {
				if (version.getName().startsWith(".")){
					continue;
				}
				
				String outputDir = outputAllDir + "/" + project.getName();//  + "/" + version.getName()
				
				if (! new File(outputDir).exists()) {
					new File(outputDir).mkdirs();
				}
				String outputFilePath = outputDir + "/" + version.getName();
				extractJavaFilesForBugLocator(version.getAbsolutePath(), outputFilePath);
			}
		}
		
	}

	public static void extractMethodsFromProject(String codeDirectory, String outputDir) throws IOException {
		System.out.println("Invoking comment extractor mehod");

		FileSystem fs = new FileSystem();

		int offset = codeDirectory.length();
		
		List<String> files = fs.getAllJavaFiles(codeDirectory, codeDirectory.length());
		Iterator <String> it = files.iterator();

		int count = 1;
		int fileCount = 0;
		System.out.println("Processing");
		while(it.hasNext()){

			fileCount++;

			String filePath = it.next();
			System.out.println("Class. " + fileCount+"\t"+filePath);
			
			String classRelativePath = filePath.substring(offset).split("\\.java")[0]; // class relative path (from the inputCodeDirectory)
			ASTParser parser = ASTParser.newParser(AST.JLS3);
			String source = FileSystem.readFile(filePath);

			parser.setSource(source.toCharArray());
			parser.setKind(ASTParser.K_COMPILATION_UNIT);
			final CompilationUnit cu = (CompilationUnit) parser.createAST(null);

			List<TypeDeclaration> classes =  cu.types();

			for(int i=0;i<classes.size();i++){ // loop through classes in the .java file
				TypeDeclaration curType = classes.get(i);
			
				String classDirectory;
				
				classDirectory = outputDir + classRelativePath;
				
//				if (className.equals(curType.getName().toString())) {
//					classDirectory = outputDir + "/" + classRelativePath;
//				} else {
//					classDirectory = outputDir + "/" + classRelativePath + "/" + curType.getName();
//				}
				
				if ( !new File(classDirectory).exists()) {
					new File(classDirectory).mkdirs();
				}
				
				
				MethodDeclaration[] methodList = curType.getMethods(); // for each class, look for its methods
				for(int j=0; j<methodList.length; j++){
					MethodDeclaration cMethod = methodList[j];

					String methodFileName = classDirectory + "/" + cMethod.getName() + ".java";

					File file = new File(methodFileName);

					System.out.println("Method. " + count++ + "\t" + methodFileName);

					// if file doesnt exists, then create it
					if (!file.exists()) {
						file.createNewFile();
					}

					FileWriter fw = new FileWriter(file.getAbsoluteFile());
					BufferedWriter bw = new BufferedWriter(fw);

					bw.write(cMethod.toString());
					bw.close();
				}

			}    
		}
		
		System.out.println("End");
	}
	
	public static void extractMethodsForMultipleProjects(String inputDir, String outputDir) throws IOException {
		File[] projects = new File(inputDir).listFiles();
		for (File project : projects) {
			if (project.getName().startsWith(".") || project.getName().equals("lucene37")) { // || new File(outputDir + "/" + project.getName()).exists() || ! project.getName().equals("aspectj")) {
				continue;
			}
			
			File[] versions = new File(project.getAbsolutePath()).listFiles();
			for (File version : versions) {
				
				if (version.getName().startsWith(".")) { // || ! version.getName().equals("ASPECTJ-100227")) {
					continue;
				}
				
				String inputProjectVersionDir = version.getAbsolutePath(); // + "/" + "pre-fix";
				String outputProjectVersionDir = outputDir + "/" + project.getName() + "/" + version.getName(); // + "/pre-fix" ;
				extractMethodsFromProject(inputProjectVersionDir, outputProjectVersionDir);
			}
		}
	}

	public static void updateFixedFilesForBR(String xmlFilePath, String filePathToNewFixedFiles, String outputFile) throws IOException {
		List<String> bugreportContent = FileHandler.readAllLines(xmlFilePath);
		List<String> fixedFiles = FileHandler.readAllLines(filePathToNewFixedFiles);
		List<String> updatedBRContent = new ArrayList<String>();
		
		for (String line : bugreportContent) {
			if (line.trim().startsWith("<file>")){
				break;
			}
			updatedBRContent.add(line);
		}
		
		for(String file : fixedFiles){
			updatedBRContent.add("\t\t\t" + file);
		}
		
		updatedBRContent.add("\t\t" + "</fixedFiles>");
		updatedBRContent.add("\t" + "</bug>");
		updatedBRContent.add("</bugrepository>");
		
		FileHandler.writeListToFile(updatedBRContent, outputFile);
	}
	
	public static void updateAllBRs(String inputBRDir, String fixedFileDir) throws IOException {
		File[] projects = new File(inputBRDir).listFiles();
		for (File project : projects){
			if (project.getName().startsWith(".")){
				continue;
			}
			
			System.out.println("Working on project : " + project.getName());
			
			File[] bugs = new File(project.getAbsolutePath()).listFiles();
			for (File bug : bugs){
				if (! bug.getName().startsWith(".")) {
					String pathToFixedFiles = fixedFileDir + "/" + project.getName() + "/" + bug.getName().split("\\.")[0];
					if (! new File(pathToFixedFiles).exists()) {
						System.out.println("corresponding fixed file does not exist");
						System.exit(0);
					}
					
					System.out.println("Working on :" + bug.getName());
					updateFixedFilesForBR(bug.getAbsolutePath(), pathToFixedFiles, bug.getAbsolutePath());
				}
			}
			
		}
	}
	
	
	public static void extracAndFlattentMethodsFromProject(String codeDirectory, String outputDir) throws IOException {
		System.out.println("Invoking comment extractor mehod");
		
		if ( !new File(outputDir).exists()) {
			new File(outputDir).mkdirs();
		}

		FileSystem fs = new FileSystem();

		int offset = codeDirectory.length();
		
		List<String> files = fs.getAllJavaFiles(codeDirectory, codeDirectory.length());
		Iterator <String> it = files.iterator();

		int count = 1;
		int fileCount = 0;
		System.out.println("Processing");
		while(it.hasNext()){

			fileCount++;

			String filePath = it.next();
			System.out.println("Class. " + fileCount+"\t"+filePath);
			
			String classRelativePath = filePath.substring(offset+1).split(".java")[0]; // class relative path (from the inputCodeDirectory)
//			String fileName = filePath.substring(offset).split("\\.")[1];
			

//			String part[]= classRelativePath.split("\\/");
//			String className = part[part.length-1];

			ASTParser parser = ASTParser.newParser(AST.JLS3);
			String source = FileSystem.readFile(filePath);

			parser.setSource(source.toCharArray());
			parser.setKind(ASTParser.K_COMPILATION_UNIT);
			final CompilationUnit cu = (CompilationUnit) parser.createAST(null);

			List<TypeDeclaration> classes =  cu.types();

			for(int i=0;i<classes.size();i++){ // loop through classes in the .java file
				TypeDeclaration curType = classes.get(i);
			
//				String classDirectory;
				
				//classDirectory = outputDir + classRelativePath;
				
//				if (className.equals(curType.getName().toString())) {
//					classDirectory = outputDir + "/" + classRelativePath;
//				} else {
//					classDirectory = outputDir + "/" + classRelativePath + "/" + curType.getName();
//				}
				
				MethodDeclaration[] methodList = curType.getMethods(); // for each class, look for its methods
				
				for(int j=0; j<methodList.length; j++){
					MethodDeclaration cMethod = methodList[j];

					String methodFileName = (classRelativePath + "/" + cMethod.getName() + ".java").replaceAll("/", ".");

					File file = new File(outputDir+ "/" + methodFileName);

					System.out.println("Method. " + count++ + "\t" + methodFileName);

					// if file doesnt exists, then create it
//					if (!file.exists()) {
//						file.createNewFile();
//					}
					
					file.createNewFile();

					FileWriter fw = new FileWriter(file.getAbsoluteFile());
					BufferedWriter bw = new BufferedWriter(fw);

					bw.write(cMethod.toString());
					bw.close();
				}

			}    
		}
		
		System.out.println("End");
	}
	
	public static void flattenFilesToXMLFixedFiles(String inputDir, String outputXMLFile) throws IOException{
		FileSystem fs = new FileSystem();

		int offset = inputDir.length();
		
		List<String> files = fs.getAllJavaFiles(inputDir, offset);
		List<String> fixedFiles = new ArrayList<String>();
		
		for (String file : files) {
			String fixedFile  = "<file>" + file.substring(offset+1) + "</file>";
			fixedFiles.add(fixedFile);
		}
		
		FileHandler.writeListToFile(fixedFiles, outputXMLFile);
		
	}
	
	public static void flattenCode(String inputDir, String outputDir) throws IOException{
		FileSystem fs = new FileSystem();

		int offset = inputDir.length();
		
		List<String> files = fs.getAllJavaFiles(inputDir, offset);
		
		if (! new File(outputDir).exists()){
			new File(outputDir).mkdirs();
		}
		
		for(String file : files) {
			String relativePath = file.substring(offset + 1).replaceAll("/", ".");
			System.out.println(file);
//			List<String> contents = FileHandler.readAllLines(file);
//			FileHandler.writeListToFile(contents, outputDir + "/" + relativePath);
			//Files.copy(file, outputDir + "/" + relativePath, REPLACE_EXISTING);
			
			File sourceFile = new File(file);
			File destFile = new File(outputDir + "/" + relativePath);
			FileUtils.copyFile(sourceFile, destFile);
			
		}
	}
	
	public static void addClassDeclare(String sourcePath, String destPath) throws IOException {
		List<String> sourceContent = FileHandler.readAllLines(sourcePath);
		if (! sourceContent.get(0).trim().contains("public class")) {
			sourceContent.add(0, "public class Foo{");
			sourceContent.add("}");
		}

		FileHandler.writeListToFile(sourceContent, destPath);
		
		System.out.println("Wrote #. " + sourceContent.size());
	}
	
	public static void addClassDeclareforAllFlattenMethods(String sourceDir, String destDir) throws IOException {
		File[] files = new File(sourceDir).listFiles();
		for (File file : files) {
			if (file.getName().startsWith(".") || ! file.getName().endsWith(".java")) {
				continue;
			}
			
			addClassDeclare(file.getAbsolutePath(), destDir + "/" + file.getName());
			
		}
	}
	
	public static void addClassDeclareforAllProjects(String inputDir, String outputDir) throws IOException {
		File[] projects = new File(inputDir).listFiles();
		for (File project : projects) {
			if (project.getName().startsWith(".")){
				continue;
			}
			
			File[] versions = new File(project.getAbsolutePath()).listFiles();
			for (File version : versions) {
				if (version.getName().startsWith(".")) {
					continue;
				}
				
				String projectInputPath = version.getAbsolutePath(); // + "/" + "pre-fix";
				String projectOutputPath = outputDir + "/" + project.getName() + "/" + version.getName(); // + "/" + "pre-fix";
				
				if (!new File(projectOutputPath).exists()) {
					new File(projectOutputPath).mkdirs();
				}
				
				System.out.println("Working on :" + project.getName() + ": " + version.getName());
				
				addClassDeclareforAllFlattenMethods(projectInputPath, projectOutputPath);
			}
		}
	}
	
	public static void flattenAllProjects(String inputDir, String outputDir) throws IOException{
		File[] projects = new File(inputDir).listFiles();
		for (File project : projects) {
			if (project.getName().startsWith(".")) {
				continue;
			}
			
//			if (! project.getName().equals("aspectj")) {
//				continue;
//			}
			
			File[] versions = new File(project.getAbsolutePath()).listFiles();
			
			for (File version : versions) {
				if (version.getName().startsWith(".")) {
					continue;
				}
//				
//				if (!version.getName().equals("ASPECTJ-88652")) {
//					continue;
//				}
				
				String inputPath = version.getAbsolutePath(); // + "/" + "pre-fix";
				String outputPath = outputDir + "/" + project.getName() + "/" + version.getName(); // + "/" + "pre-fix";
				flattenCode(inputPath, outputPath);
				
			}
		}
	}
	
	public static void generateBugReports(String originalXmlBrFilePath, String codeDir, String newXmlBrFilePath) throws IOException {
		List<String> originalXmlBrContent = FileHandler.readAllLines(originalXmlBrFilePath);
		List<String> newXmlBrContent = new ArrayList<String>();
		
		for (String line : originalXmlBrContent) {
			if (line.trim().startsWith("<file>")) {
				break;
			}
			newXmlBrContent.add(line);
		}
		
		FileSystem fs = new FileSystem();
		int offset = codeDir.length();
		List<String> javaFiles = fs.getAllJavaFiles(codeDir, offset);
		for (String javaFile : javaFiles) {
			newXmlBrContent.add("\t\t\t" + "<file>" + javaFile.substring(offset+1).replaceAll("/", ".") + "</file>");
		}
		
		newXmlBrContent.add("\t\t" + "</fixedFiles>");
		newXmlBrContent.add("\t" + "</bug>");
		newXmlBrContent.add("</bugrepository>");
		
		FileHandler.writeListToFile(newXmlBrContent, newXmlBrFilePath);

	}
	
	public static void generateBugReportsForAllProjects(String originalBrDir, String codeDir, String newBrDir) throws IOException {
		
		File[] projects = new File(originalBrDir).listFiles();
		
		for (File project: projects) {
			if (project.getName().startsWith(".")) {
				continue;
			}
			
			File[] versions = new File(project.getAbsolutePath()).listFiles();
			
			for (File version : versions) {
				if (version.getName().startsWith(".")) {
					continue;
				}
				System.out.println("Working on : " + project.getName() + "---" + version.getName());
				
				String originalXmlFilePath = version.getAbsolutePath();
				String codePath = codeDir + "/" + project.getName() + "/" +version.getName().split("\\.")[0];// + "/" + "pre-fix";
				
				if (!new File(codePath).exists()) {
					System.out.println("Something wrong: a bug report does not have a corresponding src");
					System.out.println(originalXmlFilePath);
					System.out.println(codePath);
					// System.exit(0);
					continue;
				}
				
				String newBrFilePath = newBrDir + "/" + project.getName();
				
				if (! new File(newBrFilePath).exists()) {
					new File(newBrFilePath).mkdirs();
				}
				generateBugReports(originalXmlFilePath, codePath, newBrFilePath + "/" + version.getName());
				
			}
		}
		
	}
	
	public static void removePackageDeclareForFile(String inputFilePath, String outputFilePath) throws IOException {
		List<String> content  = FileHandler.readAllLines(inputFilePath);
		List<String> newConent = new ArrayList<String>();
		for (String line : content) {
			if (! line.trim().startsWith("package")) {
				newConent.add(line);
			}
		}
		FileHandler.writeListToFile(newConent, outputFilePath);
	}
	
	public static void removePackageDeclareForAllProjects(String inputDir, String outputDir) throws IOException {
		File[] projects = new File(inputDir).listFiles();
		for (File project : projects) {
			if (project.getName().startsWith(".")){
				continue;
			}
			File[] versions = new File(project.getAbsolutePath()).listFiles();
			for (File version : versions) {
				if (version.getName().startsWith(".")) {
					continue;
				}
				File[] files = new File(version.getAbsolutePath()).listFiles();
				for (File file : files) {
					if (file.getName().startsWith(".")) {
						continue;
					}
					
					String outputFilePath = outputDir + "/" + project.getName() + "/" + version.getName();
					if (! new File(outputFilePath).exists()) {
						new File(outputFilePath).mkdirs();
					}
					
					System.out.println("Working on: " + project.getName() + " " + version.getName() + " " + file.getName());
					
					removePackageDeclareForFile(file.getAbsolutePath(), outputFilePath + "/" + file.getName());
				}
			}
		}
	}


	public static void main(String args[]) throws CoreException, IOException{

//		String codeDir = "/Users/assist/BL/swt-3.1";
//		String outputDir = "/Users/assist/BL/swt-3.1-methods";
//		
//		extractMethodsFromProject(codeDir, outputDir);
		
//		String allInputDir = "/Volumes/osxrep/CODE";
//		String allOutputDir = "/Volumes/osxrep/CODE-Method";
		
//		String allInputDir = "/Users/ass	ist/dmz/filter-code/coverage";
//		String allOutputDir = "/Users/assist/dmz/filter-code/coverage-method";
//		extractMethodsForMultipleProjects(allInputDir, allOutputDir);
		
		//extractingDataForBugLocator("/Volumes/osxrep/CODE-Method", "/Volumes/osxrep/CODE-Method-file-lists");
		
		//updateAllBRs ("/Volumes/osxrep/bugs-xml-repository-method", "/Volumes/osxrep/CODE-Method-file-lists");
		
		//extracAndFlattentMethodsFromProject("/Users/assist/dmz/tools/BugLocator/rhino/RHINO-409702-class/pre-fix", "/Users/assist/dmz/tools/BugLocator/rhino/RHINO-409702-method-flatten");
		//flattenFilesToXMLFixedFiles("/Volumes/osxrep/BugLocator/rhino/RHINO-409702-method-flatten-corrected", "/Volumes/osxrep/BugLocator/rhino/RHINO-409702-method-flatten-corrected.txt");
		
//		flattenCode("/Users/assist/dmz/tools/BugLocator/rhino/RHINO-409702-class/pre-fix/src", "/Users/assist/dmz/tools/BugLocator/rhino/RHINO-409702-class-flatten");
		//addClassDeclareforAllFlattenMethods("/Volumes/osxrep/BugLocator/rhino/RHINO-409702-method-flatten", "/Volumes/osxrep/BugLocator/rhino/RHINO-409702-method-flatten-corrected");
		
		//flattenAllProjects("/Users/assist/dmz/filter-code/coverage-method", "/Users/assist/dmz/filter-code/coverage-method-flatten");
		//flattenAllProjects("/Users/assist/dmz/filter-code/coverage", "/Users/assist/dmz/filter-code/coverage-flatten");
		
		//addClassDeclareforAllProjects("/Users/assist/dmz/filter-code/coverage-method-flatten", "/Users/assist/dmz/filter-code/coverage-method-flatten-corrected");
		
		//generateBugReportsForAllProjects("/Users/assist/dmz/ir-dataset/bugs-xml-repository", "/Users/assist/dmz/filter-code/coverage-method-flatten-corrected", "/Users/assist/dmz/filter-code/coverage-method-flatten-bugs-xml-repositories");
		//generateBugReportsForAllProjects("/Users/assist/dmz/ir-dataset/bugs-xml-repository", "/Users/assist/dmz/filter-code/coverage-flatten", "/Users/assist/dmz/filter-code/coverage-flatten-bugs-xml-repositories");
		
		//removePackageDeclareForFile("/Volumes/osxrep/BugLocator/class/src-class-flatten/rhino/RHINO-409702/pre-fix/build.test.classes.org.mozilla.javascript.drivers.JsDriver.java", "/Users/assist/Desktop/build.test.classes.org.mozilla.javascript.drivers.JsDriver.java");
		
		removePackageDeclareForAllProjects("/Users/assist/dmz/filter-code/coverage-flatten", "/Users/assist/dmz/filter-code/coverage-flatten-corrected");
	}
}

