package rtp.result;

import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.io.ObjectInputStream.GetField;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import rtp.ProgramVersion;
import rtp.utility.FileSystem;


public class RepirAPFDClass {

	/**
	 * @param args
	 * @throws IOException 
	 */
	
	
	public static void main (String[] args) throws IOException {
		ProgramVersion progVersion = new ProgramVersion();
		String basePath = "/Users/ripon/RTS/repir-dataset-and-results/results/REPiR/test-class-level";

		String versions[] = {
				"ant0.0",
				"ant3.0",
				"ant4.0",
				"ant6.0",
//				"barbecue1.5a1",
				"commons-lang3.02",
				"commons-lang3.03",
				"jaxen1.0b7",
//				"jaxen1.1b2",
				"jaxen1.1b6",
				"jaxen1.1b9",
//				"jmeter0.0",
//				"jmeter1.0",
//				"jmeter2.0",
//				"jmeter3.0",
				"joda-time0.90",
				"joda-time0.98",
				"joda-time1.10",
				"joda-time1.20",
				"mime4j0.50",
				"mime4j0.61",
				"timeandmoney3.0",
				"timeandmoney4.0",
				"xmlsec1.0",
				"xstream1.20",
				"xstream1.21",
				"xstream1.22",
				"xstream1.30",
				"xstream1.31",
				"xstream1.41"
		};
			
		String classModes[] = {
//				"HDiff",
//				"HDiff.Distinct",
//				"HDiff.Distinct.Structured",
//				"HDiff.Structured",
//				"LDiff",
//				"LDiff.Distinct",
//				"LDiff.Distinct.Structured",
//				"LDiff.Structured",
				"LDiff.Unparsed"
		};

		
		for(int modeIndex = 0; modeIndex < classModes.length; modeIndex++){
			String modePath = basePath + "/" + classModes[modeIndex];

			for(int versionIndex = 0; versionIndex < versions.length; versionIndex++){
				progVersion.initializeVersion(versions[versionIndex]);
				String faultInfoPath = progVersion.getFaultInfoFile();
				int numOfTestClasses = progVersion.getNumOfTestClasses();
				String versionPath = modePath + "/" + versions[versionIndex];
				String resultFilePath = versionPath + ".txt";
				//System.out.println(resultFilePath);
				//System.out.println(progVersion);
				double apfd = getAPFD(faultInfoPath, resultFilePath, numOfTestClasses, false);
				DecimalFormat df = new DecimalFormat("#.##");
				System.out.println("APFD: "+df.format(apfd)+"\n\n");
			}	
			
		}
	}
	
	static void add(Map<String,List<Integer>> faults, String key, Integer str){
		if(faults.containsKey(key)){
			List<Integer> testCases = faults.get(key);
			testCases.add(str);
			faults.put(key, testCases);
		}
		else{
			List<Integer> testCases = new ArrayList<Integer>();
			testCases.add(str);
			faults.put(key, testCases);
		}
	}
	
	static double print(Map<String,List<Integer>> faults, int n,boolean print){
		int i = 0;
		int tf = 0;
		for (Map.Entry<String, List<Integer>> entry : faults.entrySet()) {
		    String key = entry.getKey();
		    //System.out.println("\t"+key+"--->"+faults.get(key)+"--->"+Collections.min(faults.get(key)));
		    int min = Collections.min(faults.get(key));
		    tf = tf + min;
			if(print)
		    System.out.println(++i+"\t"+faults.get(key)+"\t"+ min);
		}
		if(print)
		System.out.println();
		
		int nm = n*faults.size();
		if(print){
		System.out.println("tf\t"+tf);
		System.out.println("nm\t"+ nm);
		}
		
		double apfd = 1 - (double) tf / (double) nm + 1/(2*(double)n);
//		if(print)
//		System.out.println("APFD\t"+apfd+"\n\n");
		
		return apfd;
	}
	
	static double getAPFD(String ftInfoPath, String ourResultPath, int numOfTestClasses, boolean print) throws IOException{
		Map<String, List<Integer>> faults = new HashMap<String, List<Integer>>();
		
		BufferedReader br = new BufferedReader(new FileReader("/Users/ripon/RTS/fault-info/"+ftInfoPath));
		String str = br.readLine();
		
		int i = 1;
		
		Map<String, Integer> faultyTestClassesPos = FaultyTestCases.getClassPositionLingming("/Users/ripon/RTS/fault-info/"+ftInfoPath, ourResultPath,print);
		
		while(str!=null){
			String parts[] = str.split("-");
			//String testCaseName = parts[0].substring(6);
			//System.out.println(testCaseName);
			String changes[] = parts[1].split(",");
			for(int j = 0; j<changes.length; j++){
				Integer pos = faultyTestClassesPos.get(""+i);
				if(pos != null){
					add(faults, changes[j], pos);
				}
			}
			str = br.readLine();
			i++;
		}
		
		br.close();
		
		return print(faults,numOfTestClasses,print);
	}
}
