package td.brclassifier;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class BrParser {
	
	public static final String stackTraceRegex = "(\\s[^\\s]+Exception[\\s|\n]*)*(\\s+at.+[(]*.+java:[0-9]+[)]*)+";
	public static final String stackTraceRegex2 = "(\\s+at.+[(].+java:[0-9]+[)])+";

	/**
	 * Extract all stack traces from an input
	 * Save the output as a list of stack traces
	 * @param input
	 * @return
	 */
	public static List<String> extractStackTraces(String input) {
		Pattern r = Pattern.compile(stackTraceRegex2);
		Matcher m = r.matcher(input);
		List<String> foundStackTraces = new ArrayList<String>();
		while (m.find()){
			foundStackTraces.add(m.group());
		}
		return foundStackTraces;
	}
	
	/**
	 * output = input - stack traces found
	 * @param input
	 * @return
	 */
	public static String eliminateStackTraces(String input) {
		Pattern r = Pattern.compile(stackTraceRegex2);
		Matcher m = r.matcher(input);
		return m.replaceAll("");
	}
	
	/**
	 * return true if the input contains
	 * a stack trace, or otherwise
	 * @param input
	 * @return
	 */
	public static boolean containsStackTrace(String input) {
		Pattern r = Pattern.compile(stackTraceRegex2);
		Matcher m = r.matcher(input);
		return m.find();
	}
	
	/**
	 * Parse an xml bug report into a list of 
	 * bug objects
	 * @param xmlFile
	 * @return
	 * @throws SAXException
	 * @throws IOException
	 */
	public static List<Bug> parseXMLBugReports(String xmlFile) throws SAXException, IOException {
		try {

			List<Bug> bugList = new ArrayList<Bug>();

			File fXmlFile = new File(xmlFile);
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder;
			dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(fXmlFile);

			doc.getDocumentElement().normalize();

			//System.out.println("Root element: " + doc.getDocumentElement().getNodeName());

			NodeList nList = doc.getElementsByTagName("bug");

			System.out.println(nList.getLength());

			for (int i=0; i< nList.getLength(); i++) {
				Node node = nList.item(i);

				//System.out.println("Current element: " + node.getNodeName());

				Node bugInfoNode = node.getChildNodes().item(1); // buginformation node

				//System.out.println(bugInfoNode.getNodeName());

				String bugID = node.getAttributes().getNamedItem("id").getTextContent(); // bug report's id

				System.out.println("Bug ID.: " + bugID);

				String summary = bugInfoNode.getChildNodes().item(1).getTextContent(); // bug report summary content
				String description = bugInfoNode.getChildNodes().item(3).getTextContent(); // bug report description field

				System.out.println("Summary: " + summary);
				System.out.println("Description: " + description);

				System.out.println("\n");

				Bug bug = new Bug(bugID, summary, description);
				bugList.add(bug);
			}

			return bugList;

		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return null;
	}
	
	/**
	 * return a list of bug ids that contain stack trace
	 * @return
	 */
	public static List<String> filterBugReportsWithStackTrace(List<Bug> bugList) {
		return null;
	}

	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

}
