package td.utils;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import rtp.utility.FileSystem;
import rtp.extractor.PreProcessor;;
public class BugReportConverter {
	
	public static List<String> readFixedFiles(String pathToFixedFile) throws IOException {
		String[] fixedFiles = FileSystem.readFile(pathToFixedFile).split("\n");
		List<String> temp = new ArrayList<String>();
		for (String f : fixedFiles) {
			if (f != null && ! f.trim().isEmpty()) {
				String tmp = "";
				if (f.contains("$")) {
					tmp = f.replaceAll("/", ".").split("\\$")[0].trim();
					System.out.println("Found: " + f);
				} else {
					tmp = f.replaceAll("/", ".").split("_")[0].trim();
				}
				
				if (tmp.contains("java.")) {
					tmp = tmp.split("java.")[1] + ".java";
				} else if (tmp.contains("test.")) {
					tmp = tmp.split("test.")[1] + ".java";
				} else if (tmp.contains("src.")) {
					tmp = tmp.split("src.")[1] + ".java";
				} else {
//					System.out.println("[error] Something wrong with the ground truth: " + pathToFixedFile + " " + f);
//					System.exit(0);
					tmp += ".java";
				}
				temp.add(tmp);
			}
		}
		return new ArrayList<String>(new HashSet<String>(temp));
	}
	
	public static void composeBugReportXML(String projectID, String bugID, String pathToBR, String pathToFixedFiles, String outputXMLFile) throws IOException{
		String brContent = FileSystem.readFile(pathToBR);
		String[] parts = brContent.split(":::");
		System.out.println(parts.length);
		
		String bugId = pathToBR.split("/")[pathToBR.split("/").length - 1];
		
		String title = parts[1].trim();
		String description = parts[2].trim();
		
		List<String> fixedFiles = readFixedFiles(pathToFixedFiles);	
		
		if (fixedFiles == null || fixedFiles.isEmpty() || fixedFiles.size() == 0) {
			System.out.println("Something wrong");
			System.exit(0);
		}
	
		FileWriter fw = new FileWriter(outputXMLFile);
		BufferedWriter bw = new BufferedWriter(fw);
		bw.write("<bugrepository name=\"" + projectID + "\"" + ">");
		bw.newLine();
		
		bw.write("\t<bug id=\"" + bugId.split("-")[1].trim() +  "\" opendate=\"\" fixdate=\"\">");
		bw.newLine();
			
		bw.write("\t\t<buginformation>");
		bw.newLine();
		
		bw.write("\t\t\t<summary>");
		bw.newLine();
		
		bw.write("\t\t\t\t" + PreProcessor.process(title));
		bw.newLine();
		
		bw.write("\t\t\t</summary>");
		bw.newLine();
		
		bw.write("\t\t\t<description>");
		bw.newLine();
		
		bw.write("\t\t\t\t" + PreProcessor.process(description));
		bw.newLine();
		
		bw.write("\t\t\t</description>");
		bw.newLine();
		
		bw.write("\t\t</buginformation>");
		bw.newLine();
		
		bw.write("\t\t<fixedFiles>");
		bw.newLine();
		
		for (String f : fixedFiles) {
			bw.write("\t\t\t<file>");
			bw.write(f);
			bw.write("</file>");
			bw.newLine();	
			
		}
		
		bw.write("\t\t</fixedFiles>");
		bw.newLine();
		
		bw.write("\t</bug>");
		bw.newLine();
		
		bw.write("</bugrepository>");
		bw.newLine();
		
		bw.close();	
		fw.close();
	}
	
	public static void mergeBugReports(String projectName, List<String> list, String xmlFilePath) throws IOException {
		FileWriter fw = new FileWriter(xmlFilePath);
		BufferedWriter bw = new BufferedWriter(fw);
		bw.write("<bugrepository name=" + "\"" + projectName + "\"" + ">");
		bw.newLine();
		
		for (String elem : list) {
			bw.write(elem);
			bw.newLine();
			bw.newLine();
		}
		
		bw.write("</bugrepository>");
		bw.newLine();
		
		bw.close();	
		fw.close();
	}
	
	public static void convertBugReports(String bugFolderPath, String fixedFilesFolderPath, String outputPath) throws IOException {
		File directory = new File(bugFolderPath);
		File[] files = directory.listFiles();
		
		for (File f : files) {
			if (f.isDirectory() && f.getName().toLowerCase().contains("aspectj")) {
				String projectName = f.getName();
				System.out.println(projectName);
				
				File dir = new File(outputPath + "/" + projectName);
				dir.mkdir(); // create a corresponding output folder
				
				File[] bugReports = f.listFiles();
				for (File br : bugReports) {
					String bugId = br.getName();
					//System.out.println(bugId);
					
					String pathToBR = br.getAbsolutePath();
					System.out.println(pathToBR);
					
					String pathToFixedFiles = fixedFilesFolderPath + "/" + projectName + "/" + bugId + ".txt";
					System.out.println(pathToFixedFiles);
					
					String pathToOutputFile = dir.getAbsolutePath() + "/" + bugId + ".xml";
					System.out.println(pathToOutputFile);
					
					composeBugReportXML(projectName, bugId, pathToBR, pathToFixedFiles, pathToOutputFile);
				}
			}
		}
	}
	
	public static void createBugRepository(String bugDirPath, String outDir) throws IOException {
		File bugDir = new File(bugDirPath);
		File[] projects = bugDir.listFiles();
		
		for (File p : projects) {
			List<String> elems = new ArrayList<String>();
			if (p.isDirectory()) {
				System.out.println("Working on " + p.getName());
				File[] xmlFiles = p.listFiles();
				for (File xmlFile : xmlFiles) {
					String content = FileSystem.readFile(xmlFile.getAbsolutePath()).trim();
					elems.add(content);
				}
				mergeBugReports(p.getName(), elems, outDir + "/" + p.getName() + ".xml");
			}
		}
		
		System.out.println("done");
	}
	
	public static void main(String[] args) throws IOException {
		// TODO Auto-generated method stub
		String br = "/Users/assist/dmz/ir-dataset/bugs/bug-reports";
		String fixed = "/Users/assist/dmz/ir-dataset/ground_truth";
		String outputFile = "/Users/assist/dmz/ir-dataset/bugs-xml-repository";
		String tempDir = "/Users/assist/dmz/ir-dataset/bugs-xml-repository";
		convertBugReports(br, fixed, outputFile);
		//createBugRepository(outputFile, tempDir);
	}

}
