package td.utils;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;	

public class Clover {
	
	public static Document readXML(String filePath) {
		try {
			File fXmlFile = new File(filePath);
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			Document doc = dBuilder.parse(fXmlFile);
			return doc;
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println(filePath);
			return null;
		}
	}
	
	/**
	 * Convert Clover coverage data in xml to
	 * class (file) coverage format
	 * @param coverageXMLFile
	 * @return
	 */
	public static List<String> cloverToClassCV(String coverageXMLFile) {
		Document doc = readXML(coverageXMLFile);
		NodeList fileNodes = doc.getElementsByTagName("file");
		List<String> coveredClasses = new ArrayList<String>();
		
		for (int i=0; i<fileNodes.getLength(); i++) {
			Node file = fileNodes.item(i);
			String fileFullName = file.getAttributes().getNamedItem("path").getNodeValue();
			Element elem = (Element) file;
			int coveredMethods = Integer.parseInt(elem.getElementsByTagName("metrics").item(0).getAttributes().getNamedItem("coveredmethods").getNodeValue());
			if (coveredMethods > 0) {
				coveredClasses.add(fileFullName.split("post-fix/")[1].trim()); // + ":" + coveredMethods);
			}
		}
		return coveredClasses;
	}
	
	/**
	 * Convert coverage data in xml to 
	 * method coverage format 
	 * @param coverageXMLFile
	 * @return
	 */
	public static List<String> cloverToMethodCV(String coverageXMLFile) {
		Document doc = readXML(coverageXMLFile);
		NodeList fileNodes = doc.getElementsByTagName("file");
		List<String> coveredMethods = new ArrayList<String>();

		for (int i=0; i<fileNodes.getLength(); i++) {
			Node file = fileNodes.item(i);
			String fileFullName = file.getAttributes().getNamedItem("path").getNodeValue().split("post-fix/")[1].trim().split(".java")[0];
			Element elem = (Element) file;
			NodeList lines = elem.getElementsByTagName("line");
			for (int j=0; j<lines.getLength(); j++){
				Element line = (Element) lines.item(j);
				String lineType = line.getAttributes().getNamedItem("type").getNodeValue().trim();

				if (lineType.equals("method")){
					int lineCount = Integer.parseInt(line.getAttributes().getNamedItem("count").getNodeValue().trim());

					if (lineCount > 0){

						String methodName = line.getAttributes().getNamedItem("signature").getNodeValue().trim().split("\\(")[0];
						if (methodName.contains(",")){
							methodName = methodName.split(",")[0].trim();
						}
						
						coveredMethods.add(fileFullName.trim().replace("/", ".") + "." + methodName.trim());
					}

				}
			}
		}

		return coveredMethods;

	}
	
	public static void printList(List<String> list) {
		int i=0;
		for(String elem : list) {
			System.out.println(++i + ". " + elem);
		}
		System.out.println("done");
	}
	
	/**
	 * Convert all to class coverage
	 * @param dir
	 * @param dest_dir
	 */
	public static void convertAllToClassCV(String dir, String dest_dir){

		File[] ids = new File(dir).listFiles(File::isDirectory);
		
		int count = 0;

		for (File id : ids) {
			String xmlPath = id.getAbsolutePath() + "/" + "coverage-" + id.getName() + ".xml";
			if (new File(xmlPath).exists()) {
				List<String> coveredClasses = cloverToClassCV(xmlPath);
				String txtPath = dest_dir + "/" + id.getName();
				try {
					System.out.println("[" + ++count + "]" + " Writing file: " + id.getName() + ".mcv");
					FileHandler.writeListToFile(coveredClasses, txtPath);
					printList(coveredClasses);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					System.out.println(id);
				}
			}
		}

		System.out.println("There are total written files: " + count);
	}
	
	/**
	 * Convert all to Method CV
	 * @param dir
	 * @param dest_dir
	 */
	public static void convertAllToMethodCV(String dir, String dest_dir) throws IOException{
		File[] ids = new File(dir).listFiles(File::isDirectory);
		
		int count = 0;

		for (File id : ids) {
			String xmlPath = id.getAbsolutePath() + "/" + "coverage-" + id.getName() + ".xml";
			if (new File(xmlPath).exists()) {
				List<String> coveredMethods= cloverToMethodCV(xmlPath);
				String txtPath = dest_dir + "/" + id.getName() + ".mcv";
				System.out.println("[" + ++count + "]" + " Writing file: " + id.getName() );
				FileHandler.writeListToFile(coveredMethods, txtPath);
				printList(coveredMethods);
			}
		}

		System.out.println("There are total written files: " + count);
	}
	
	public static void collectTestExecutionsForAspectJFromClover(String inputDir, String outputDir) throws IOException {
		File[] versions = new File(inputDir).listFiles();
		for (File version : versions) {
			if (version.exists() && version.isDirectory() && ! version.getName().startsWith(".")){
				
				if (!version.getName().equals("ASPECTJ-118781")) {
					continue;
				}
				
				File[] tests = new File(version.getAbsolutePath()).listFiles();
				List<String> version_traces = new ArrayList<String>();
				System.out.println("Working on ... " + version.getName());
				for (File test : tests) {
					if (test.exists() && test.isFile() && test.getName().endsWith(".xml")){
						String testName = "";
						String testResult = "";
						if (test.getName().contains("passed")){
							testName = test.getName().split("passed_")[1].split("\\.")[0].trim();
							testResult = "passed";
						} else if (test.getName().contains("failed")) {
							testName = test.getName().split("failed_")[1].split("\\.")[0].trim();
							testResult = "failed";
						} else {
							System.out.println("Something wrong with the test becase it must be either failed or passed");
							System.out.println(version.getName() + " " + test.getName());
							System.exit(0);
						}
						System.out.println("Working on ..." + test.getName());
						List<String> traces = cloverToMethodCV(test.getAbsolutePath());
						traces.add(0,"<Test>" + testName + ":" + testResult);
						
						//FileHandler.appendListToFile(outputDir + "/" + version.getName() + ".txt", traces);
						version_traces.addAll(traces);
					}
				}
				if (new File(outputDir + "/" + version.getName() + ".txt").exists()) {
					System.out.println("File already existed: " + version.getName() + ".txt");
				} else {
					FileHandler.writeListToFile(version_traces, outputDir + "/" + version.getName() + ".txt");
				}
				
			}
		}
	}

	public static void main(String[] args) throws IOException {
		// TODO Auto-generated method stub
//		
//		String xmlFile = "/Users/assist/dmz/gen-data/clover/aspectj/100227/coverage-100227.xml";
//		printList(cloverToClassCV(xmlFile));
		
		String dir = "/Users/assist/dmz/gen-data/clover/aspectj";
		String dest_dir = "/Users/assist/dmz/gen-data/faulttracer/aspectj/post-fix/coverage-data/union/class";
		
		//convertAllToMethodCV(dir, dest_dir);

		//convertAllToClassCV(dir, dest_dir);
		
		//printList(cloverToMethodCV("/Users/assist/dmz/gen-data/Spectrum/4SP/clover-aspectj/ASPECTJ-100227/coverage_passed_100227_3.xml"));
		//collectTestExecutionsForAspectJFromClover("/Users/assist/dmz/gen-data/Spectrum/4SP/clover-aspectj", "/Users/assist/dmz/gen-data/Spectrum/4SP/sp/aspectj");
		//collectTestExecutionsForAspectJFromClover(args[0], args[1]);
		
		collectTestExecutionsForAspectJFromClover("/Users/assist/dmz/gen-data/Spectrum/4SP/clover-aspectj", "/Users/assist/IR-SP-combined/sp-data/aspectj");

	}

}
