package td.utils;

import java.io.File;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class ProjectSpectrum {
	
	private String spectraFile;
	private List<Testcase> tests;
	private int totalFailedTests = 0;
	private int totalPassedTests = 0;
	private int totalTests = 0;
	
	public static void methodNameCorrection(String inputDir, String outputDir) throws IOException {
		File[] files = new File(inputDir).listFiles();
		
		for (File f : files) {
			if (f.exists() && f.isFile() && (! f.getName().contains("DS_Store"))){
				List<String> methodNames = new ArrayList<String>();
				List<String> lines = FileHandler.readAllLines(f.getAbsolutePath());
				for (String line : lines) {
					if (line.contains("<DOCNO>")) {
						String methodName = line.split(">")[1].trim().split("<")[0].trim();
						//System.out.println(methodName);
//						if (methodName.contains("org.mozilla")) { // rhiho case
//							methodName = "org.mozilla" + methodName.split("org.mozilla")[1];
//						} else if (methodName.contains("org.apache")) { //ant case
//							methodName = "org.apache" + methodName.split("org.apache")[1];
//						} else if (methodName.contains("org.apache.lucene")) { // lucene case
//							methodName = "org.apache.lucene" + methodName.split("org.apache.lucene")[1];
//						} else if (methodName.contains("org.aspectj")) {
//							System.out.println("aspectj item");
//						}
						
						methodNames.add(methodName);
						
					}
				}
				System.out.println(methodNames.size());
				HashSet<String> uniqueMethodNames = new HashSet<>(methodNames);
				System.out.println(uniqueMethodNames.size());
				
				String outputPath = outputDir + "/" + f.getName();
				
				List<String> list = new ArrayList<String>();
				for (String name : uniqueMethodNames){
					list.add(name);
				}
				
				FileHandler.writeHashToFile(uniqueMethodNames, outputPath);
				
			}
			
		}
	}
	
	public boolean failedTestExists() {
		if (tests != null ){
			for (Testcase t : tests) {
				if (t.getResult().contains("failed")) {
					System.out.println(t.getName());
					return true;
				}
			}
		}
		return false;
	}
	
	public void printTests() {
		for (Testcase t : tests) {
			System.out.println(t.getName() + " - " + t.getResult());
		}
	}
	
	public List<String> getSpectrumDataForAllMethods(String methodsPath, String level) throws IOException {
		List<String> lines = FileHandler.readAllLines(methodsPath);
		List<String> spectra = new ArrayList<String>();
		int count = 0;
		for (String line : lines) {
//			Spectrum spectrum = getSpectrumOfAnElem(line.split(" ")[0].trim(), level);// line is in format: [item_name] [score]
			
			Spectrum spectrum = getSpectrumOfAnElem(line, level);
			
			String spectrumLine = line + " ef="+spectrum.getEf() + " ep=" + spectrum.getEp() + " nf=" + spectrum.getNf() + " np=" + spectrum.getNp();
			spectra.add(spectrumLine);
			System.out.println(++count + ". " + spectrumLine);
		}
		System.out.println(lines.size());
		System.out.println(spectra.size());
		return spectra;
	}
	
	ProjectSpectrum (String spDataFile, String amlTestList) throws IOException {
		spectraFile = spDataFile;
		if (spDataFile.contains("aspectj")){
			tests = getTestsFromClover(spDataFile);
		} else {
			tests = getTestsFromFaulttracer(spDataFile);
			correctFailedTestsUsingAML(amlTestList);
		}
		
		countTotalFPTests();
		
		if (! failedTestExists() || (totalTests == 0)) {
			System.out.println("This project version: " + spDataFile + "does not contain failed tests ==> invalid");
			System.exit(0);
		}
	}
	
	public List<String> getFailedTestsFromAML(String amlTestListFile) throws IOException{
		List<String> lines = FileHandler.readAllLines(amlTestListFile);
		List<String> failedTests = new ArrayList<String>();
		for (int i=1; i<lines.size(); i++){
			if (lines.get(i).split(",")[2].equals("failed")) {
				String failedTestName = lines.get(i).split(",")[0] + "." + lines.get(i).split(",")[1] ;
				failedTests.add(failedTestName);
			}
		}
		return failedTests;
	}
	
	public void correctFailedTestsUsingAML(String amlTestListFile) throws IOException{
		List<String> failedTests = getFailedTestsFromAML(amlTestListFile);
		boolean corrected = false;
		for (String t: failedTests) {
			for (Testcase tc : tests) {
				if (tc.getName().equals(t)) {
					tc.setResult("failed");
					System.out.println(tc.getName() + " has been corrected to failed");
					corrected = true;
				}
			}
		}
		
		if (! corrected){
			System.out.println("[error] no failed tests found" );
			System.out.println(amlTestListFile);
			//System.exit(0);
		}
			
	}
	
	public List<Testcase> getTests(){
		return tests;
	}
	
	private void countTotalFPTests() {
		if (tests == null) {
			return;
		}
		
		for (Testcase t : tests){
			if (t.getResult().equals("passed")) {
				totalPassedTests += 1;
			} else {
				totalFailedTests += 1;
			}
		}
		totalTests = totalPassedTests + totalFailedTests;
		
		System.out.println("failed tests: " + totalFailedTests);
		System.out.println("passed tests: " + totalPassedTests);
		
		if (totalFailedTests < 1) {
			System.out.println("Something wrong. There must be at least 1 failed test");
			System.out.println(spectraFile);
		}
	}
	
	public static boolean isElementCovered2(List<String> coveredElems, String elem, String level){		
		//for method case
		if (level.equals("method")){
			for (String e : coveredElems){
				if (elem.endsWith(e) || e.endsWith(elem)){
					return true;
				}
			}
		}
		// for class case
		if (! elem.endsWith(".java")) {
			elem += ".java";
		}
		
		for (String e : coveredElems){
			if (elem.endsWith(getClassName2(e)) || getClassName2(e).endsWith(elem)) {
				return true;
			}
			
		}
		
		return false;
	}
	
	public static String getClassName2(String line){
		String[] parts = line.split("\\.");
		String className = "";
		for (String part : parts){
			if (Character.isUpperCase(part.charAt(0))) {
				className += part;
				break;
			} else {
				className += part + ".";
			}
		}
		if (className.contains("$")){
			className = className.split("\\$")[0];
		}
		
		if (! className.endsWith(".java")) {
			className += ".java";
		}
		return className;
	}
	
	public static boolean isElementCovered(List<String> coveredElems, String elem, String level){
//		if (level.equals("method")) {
//			//return coveredElems.contains(elem);
//		} 
		
		//for method case
		if (level.equals("method")){
			for (String e : coveredElems){
				if (elem.endsWith(e) || e.endsWith(elem)){
					return true;
				}
			}
		}
		// for class case
		for (String e : coveredElems){
//			if (e.startsWith(elem + ".") || e.startsWith(elem + "$")) {
//				return true;
//			}
			
			if (elem.endsWith(getClassName(e)) || getClassName(e).endsWith(elem)) {
				return true;
			}
			
		}
		
		return false;
	}
	
	public static String getClassName(String line){
		String[] parts = line.split("\\.");
		String className = "";
		for (String part : parts){
			if (Character.isUpperCase(part.charAt(0))) {
				className += part;
				break;
			} else {
				className += part + ".";
			}
		}
		if (className.contains("$")){
			className = className.split("\\$")[0];
		}
		return className;
	}
	
	public Spectrum getSpectrumOfAnElem(String elemName, String level) {
		if (tests == null) return null;
		int ep = 0; //executed passing tests
		int ef = 0; //executed failing tests
		int nf = 0; //non-executed failing tests
		int np = 0; //non-executed passing tests
		
		for (Testcase t: tests) {
			List<String> coveredElems = t.getExecutedElement();
			//if (coveredElems.contains(elemName)) { // this condition is for method level
			//if (isElementCovered(coveredElems, elemName)) { // this is for class level
			
			if(isElementCovered2(coveredElems, elemName, level)){
				//System.out.println("Test: " + t.getName() + " executed " + elemName);
				if (t.getResult().equals("passed")){
					ep += 1;
				} else {
					ef += 1 ;
				}
			}		
		}
		
		np = totalPassedTests - ep;
		nf = totalFailedTests - ef;
		
		Spectrum spectrum = new Spectrum();
		spectrum.setElemName(elemName);
		spectrum.setEf(ef);
		spectrum.setEp(ep);
		spectrum.setNf(nf);
		spectrum.setNp(np);
		
		return spectrum;

	}
	
	/**
	 * Extract tests from colver data file, used for AspectJ case
	 * @param cloverDataFile
	 * @return
	 */
	private List<Testcase> getTestsFromClover(String cloverDataFile){
		List<Testcase> testcases = new ArrayList<Testcase>();
		try {
			List<String> lines = FileHandler.readAllLines(cloverDataFile);
			
			String testName = "";
			String testResult = "";
			List<String> coveredElems = null;
			
			int maxTests = 101; // limit the number of test is 100
			int countTests = 0;
			
			for (String line : lines){
				if (line.startsWith("<Test>")){
					Testcase test = new Testcase(testName, testResult, coveredElems);
					testcases.add(test);
					
					testName = line.split(":")[0].trim();
					testResult = line.split(":")[1].trim();
					coveredElems = new ArrayList<String>();
					
					countTests ++;
				} else {
					coveredElems.add(line);
				}
				
				if (countTests > maxTests) {
					break;
				}
			}
			
			Testcase test = new Testcase(testName, testResult, coveredElems);
			testcases.add(test);
			
			testcases.remove(0);
			
		} catch (Exception e) {
			System.out.println(e.getStackTrace());
			System.out.println("something wrong when processing file: " + cloverDataFile);
		}
		
		return testcases;
	}
	
	
	
	/**
	 * For Methods
	 * @param faulttracerOutputFile
	 * @return
	 */
	private  List<Testcase> getTestsFromFaulttracer(String faulttracerOutputFile) {
		try {
			System.out.println("Working on : " + faulttracerOutputFile);
			List<String> lines = FileHandler.readAllLines(faulttracerOutputFile);
			
			List<Testcase> testcases = new ArrayList<Testcase>();

			String testName = "";
			String testResult = "";
			List<String> coveredElems = null;
			
			for (String line : lines) {
				if (line.contains("[java]")) {
					if (line.contains("<Test>")) {
						Testcase test = new Testcase(testName, testResult, coveredElems);	
						testcases.add(test);
						
						coveredElems = new ArrayList<String>();
						testName = line.split("<Test>")[1].trim();
						
						if (testName.endsWith("false")) {
							testName = testName.split("-false")[0].trim();
							testResult = "passed";
						} else {
							testName = testName.split("-true")[0].trim();
							testResult = "passed";
						}
		
						//System.out.println(testName);
						//System.out.println(testResult);
					} else { // if (! line.split("\\[java\\]")[1].trim().contains("WARN") && ! line.split("\\[java\\]")[1].trim().startsWith("Exception") && ! line.split("\\[java\\]")[1].trim().startsWith("at")) {
						//System.out.println(line);
						String coveredElem = line.trim().split(":")[0].split(" ")[1];
						//System.out.println(coveredElem);
						coveredElems.add(coveredElem);
					} 
//					else {
//						System.out.println("line does not contain execution path");
//						System.out.println(line);
//					}
				}
			}
			
			Testcase test = new Testcase(testName, testResult, coveredElems);	
			testcases.add(test);
			
			testcases.remove(0);
			
			return testcases;
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println(faulttracerOutputFile);
			return null;
		}
	}
	
	/**
	 * For Methods
	 * @param faulttracerOutputFile
	 * @return
	 */
	private  List<Testcase> getTestsFromFaulttracerClassLevel(String faulttracerOutputFile) {
		try {
			System.out.println("Working on : " + faulttracerOutputFile);
			List<String> lines = FileHandler.readAllLines(faulttracerOutputFile);
			
			List<Testcase> testcases = new ArrayList<Testcase>();

			String testName = "";
			String testResult = "";
			List<String> coveredElems = null;
			
			for (String line : lines) {
				if (line.contains("[java]")) {
					if (line.contains("<Test>")) {
						Testcase test = new Testcase(testName, testResult, coveredElems);	
						testcases.add(test);
						
						coveredElems = new ArrayList<String>();
						testName = line.split("<Test>")[1].trim();
						
						if (testName.endsWith("false")) {
							testName = testName.split("-false")[0].trim();
							testResult = "failed";
						} else {
							testName = testName.split("-true")[0].trim();
							testResult = "passed";
						}
		
						//System.out.println(testName);
						//System.out.println(testResult);
					} else {
						//System.out.println(line);
						String coveredElem = line.trim().split(":")[0].split(" ")[1];
						//System.out.println(coveredElem);
						coveredElems.add(coveredElem);
					}
				}
			}
			
			Testcase test = new Testcase(testName, testResult, coveredElems);	
			testcases.add(test);
			
			testcases.remove(0);
			
			return testcases;
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println(faulttracerOutputFile);
			return null;
		}
	}
		
	public void writeTracesToFiles (String filePath) throws IOException {
		List<String> traces = new ArrayList<String>();
		for (Testcase t : tests) {
			traces.add("[Test] " + t.getName() + " - " + t.getResult());
			for (String method : t.getExecutedElement()) {
				traces.add("[java]" + method);
			}
			traces.add("\n");
		}
		FileHandler.writeListToFile(traces, filePath);
	}
	
	public void printTraces () {
		for (Testcase t : tests) {
			System.out.println(t.getName() + " - " + t.getResult());
			for (String method : t.getExecutedElement()) {
				System.out.println(method);
			}
			System.out.println("----------------------------");
		}
	}
	/**
	 * Compute spectrum for a project whose test executions 
	 * are generated by faulttracer
	 * @param faulttracerProjectDir: faulttracer coverage data
	 * @param amlTestListDir: list test results by David Lo
	 * @param methodListDir : item to be sp computed (either class or method)
	 * @param outputSpectraPath: output directory
	 * @param level: is item method or class
	 * @throws IOException
	 */
	public static void computeSPforProject(String spDataDir, String amlTestListDir, String itemListDir, String outputSpectraPath, String level) throws IOException {
		File[] files = new File(spDataDir).listFiles();
		
		for (File f : files) {
			if (! f.getName().startsWith(".") && ! new File(outputSpectraPath + "/" + f.getName()).exists()){
				System.out.println("Working on ... " + f.getName());
				
				String spDataFilePath = f.getAbsolutePath();
				String amlTestListPath = amlTestListDir + "/" + f.getName();
				//String outputPath = outputDir +  "/" + f.getName();
				
				ProjectSpectrum ps = new ProjectSpectrum(spDataFilePath, amlTestListPath);
				//ps.writeTracesToFiles(outputPath);
				
				String itemListPath = itemListDir + "/" + f.getName();
				
				if (! new File(itemListPath).exists()) {
					if (f.getName().endsWith(".txt")) {
						itemListPath = itemListDir + "/" + f.getName().split("\\.")[0];
					} else {
						itemListPath = itemListDir + "/" + f.getName() + ".txt";
					}
				}
				
				List<String> spectrum = ps.getSpectrumDataForAllMethods(itemListPath, level);
			
				if (! new File(outputSpectraPath).exists()){
					new File(outputSpectraPath).mkdirs();
				}
				
				String outputMethodSpectrumPath = outputSpectraPath + "/" + f.getName();
				FileHandler.writeListToFile(spectrum, outputMethodSpectrumPath);
			}
		}
	}
	
	/**
	 * Matching IR scores with SP scores 
	 * @param spDir
	 * @param irDir
	 * @param combinedDir
	 * @throws IOException
	 */
	public static void combineSPWithIR (String irDir, String spDir, String combinedDir) throws IOException {
		
		//make sure the outputDir exists
		if (! new File(combinedDir).exists()) {
			new File(combinedDir).mkdirs();
		}
		
		//traversal the irDir
		File[] files = new File(irDir).listFiles();
		
		for (File f : files) { // loop through ir directory
			if (f.exists() && f.isFile() && ! f.getName().startsWith(".")) {
				System.out.println("working on " + f.getName());
				
				// for each ir file, get the corresponding sp file
				String spFileName = spDir + "/" + f.getName();
				
				//the corresponding sp file might not end with .txt
				//just make sure the corresponding sp file exist
				if (! new File(spFileName).exists()) {
					if (spFileName.endsWith(".txt")) {
						spFileName = spFileName.split("\\.")[0];
					} else {
						spFileName += ".txt";
					}
				}
				
				if (! new File(spFileName).exists()) {
					System.out.println("The SP corresponding file does not exist");
					//System.exit(0);
					continue;
				}
				
				//combined score list
				List<String> spCombinedWithIR = new ArrayList<String>();
				
				List<String> irLines = FileHandler.readAllLines(f.getAbsolutePath());
				
				int count = 0;
				for (String line : irLines) {
					//extract ir name and ir score. MAKE SURE each line has the same format: IR_NAME IR_SCORE
					String irItemName = line.split(" ")[0].trim();
					String irItemScore = line.split(" ")[1].trim();
					// go to find corresponding SP score
					String spScore = findSPScore(spFileName, irItemName);
					if (spScore != null) {
						String combinedIrSpLine = irItemName + " " + spScore + " " + irItemScore;
						spCombinedWithIR.add(combinedIrSpLine);
					} else {
						System.out.println(++count + ". " + line + " is not found in indexed IR docs");
						//System.out.println(++count);
					}
				}
				
				count = 0;
				
				String outputhFilePath = combinedDir + "/" + f.getName();
				FileHandler.writeListToFile(spCombinedWithIR, outputhFilePath);
				
				System.out.println("Done with " + f.getName() + "===>"+ spCombinedWithIR.size() + " items are found and saved");
				System.out.println("--------------");
			}
		}
		
	}
	
	public static String findIRScore (String indexDir, String matchingDoc) throws IOException {
		List<String> indexedDocs = FileHandler.readAllLines(indexDir);
		for (String doc : indexedDocs) {
			String indexedDocName = doc.split(" ")[2].trim();
			if (indexedDocName.endsWith(matchingDoc) || matchingDoc.endsWith(indexedDocName)) {
				//System.out.println("found");
				String score = doc.split(" ")[4].trim();
				return score;
			}
		}
		return null;
	}
	
	public static String findSPScore (String spFilePath, String matchingItem) throws IOException {
		List<String> spItems = FileHandler.readAllLines(spFilePath);
		
		if (!matchingItem.endsWith(".java")) {
			matchingItem += ".java";
		}
		
		for (String item : spItems) {
			String rankedSpItem = item.split(" ")[0].trim();
			//make sure rankedSpItem name ends with .java for consistently comparing
			if (! rankedSpItem.endsWith(".java")) {
				rankedSpItem += ".java";
			}
			
			if (rankedSpItem.endsWith(matchingItem) || matchingItem.endsWith(rankedSpItem)) {
				//System.out.println("found");
				String[] scores = item.split(" ");
				String scoreLine = scores[1] + " " + scores[2] + " " + scores[3] + " " + scores[4];
				return scoreLine;
			}
		}
		//if an irItem does not have a corresponding spScore, give it a default
		String defaultSpScore = "ef=0 ep=0 nf=1 np=99";
		return defaultSpScore;
	}
	
	public static void computeCombinedScore(String inputDir, String outputDir) throws IOException {
		File[] files = new File(inputDir).listFiles();
		
		String[] metrics = {"T", "J", "A", "O"};
		
		for (String metric : metrics) {
			
			String outputPath = "";

			switch (metric){
			case "T":
				outputPath = outputDir + "/" + "tarantula/";
				break;
			case "J":
				outputPath = outputDir + "/" + "jaccard/" ;
				break;
			case "A":
				outputPath = outputDir + "/" + "ample/";
				break;
			case "O":
				outputPath = outputDir + "/" + "ochiai/";
				break;
			default:
				System.out.println("invalid metric value");
				System.exit(0);
			}
			
			new File(outputPath).mkdirs();
			
			for (File f : files) {
				if (f.exists() && f.isFile() && ! f.getName().startsWith(".")){
					List<String> lines = FileHandler.readAllLines(f.getAbsolutePath());

					List<String> newLines = new ArrayList<String>();

					// scale ir score into the range [0 1]
					double irMin = 100000000, irMax = 0;
					for (String line : lines) {
						double irScore = Double.parseDouble(line.split(" ")[5].split("=")[1]); // get ir Score
						if (irScore>irMax) {
							irMax = irScore;
						}
						if (irMin > irScore) {
							irMin = irScore;
						}
					}

					for (String line : lines) {
						double irScore = Double.parseDouble(line.split(" ")[5].split("=")[1]);
						String irMethod = line.split(" ")[5].split("=")[0].trim();
						double irStandardizedScore = (irScore - irMin)/(irMax-irMin);
						String ef = line.split(" ")[1].split("=")[1];
						String ep = line.split(" ")[2].split("=")[1];
						String nf = line.split(" ")[3].split("=")[1];
						String np = line.split(" ")[4].split("=")[1];

						double spMetric = SPFormula.getMetric(metric, ef, ep, nf, np);
						
						if (spMetric < 0) {
							System.out.println("Metric value is incorrect");
							System.exit(0);
						}

						line += " " + irMethod.toUpperCase() + "=" + Math.floor(irStandardizedScore*1000000)/1000000;
						line += " " + metric + "=" + Math.floor(spMetric*1000000)/1000000;

						double[] combinedScores = new double[11]; 
						for (int i=0; i<11; i++){
							double a = 0.1 * i;
							combinedScores[i] = a*spMetric + (1-a)*irStandardizedScore;
							line += " " + "comb" + i + "=" + Math.floor(combinedScores[i]*1000000)/1000000;
						}

						newLines.add(line);

						System.out.println(line);

						//System.exit(0);
					}
					
					FileHandler.writeListToFile(newLines, outputPath + "/" + f.getName());
				}
			}
			
		}
	}
	
	public static void computeCombinedScoreForAspectj(String inputDir, String outputDir) throws IOException {
		File[] files = new File(inputDir).listFiles();
		
		String[] metrics = {"T", "J", "A", "O"};
		
		for (String metric : metrics) {
			
			String outputPath = "";

			switch (metric){
			case "T":
				outputPath = outputDir + "/" + "tarantula/";
				break;
			case "J":
				outputPath = outputDir + "/" + "jaccard/" ;
				break;
			case "A":
				outputPath = outputDir + "/" + "ample/";
				break;
			case "O":
				outputPath = outputDir + "/" + "ochiai/";
				break;
			default:
				System.out.println("invalid metric value");
				System.exit(0);
			}
			
			new File(outputPath).mkdirs();
			
			for (File f : files) {
				if (f.exists() && f.isFile() && ! f.getName().startsWith(".")){
					List<String> lines = FileHandler.readAllLines(f.getAbsolutePath());

					List<String> newLines = new ArrayList<String>();

					double irMin = 100000000, irMax = 0;
					for (String line : lines) {
						double irScore = Double.parseDouble(line.split(" ")[1].split("=")[1]);
						if (irScore>irMax) {
							irMax = irScore;
						}
						if (irMin > irScore) {
							irMin = irScore;
						}
					}

					for (String line : lines) {
						double irScore = Double.parseDouble(line.split(" ")[1].split("=")[1]);
						double irStandardizedScore = (irScore - irMin)/(irMax-irMin);
						String ef = line.split(" ")[2].split("=")[1];
						String ep = line.split(" ")[3].split("=")[1];
						String nf = line.split(" ")[4].split("=")[1];
						String np = line.split(" ")[5].split("=")[1];

						double spMetric = SPFormula.getMetric(metric, ef, ep, nf, np);
						
						if (spMetric < 0) {
							System.out.println("Metric value is incorrect");
							System.exit(0);
						}

						line += " " + "IR=" + Math.floor(irStandardizedScore*1000000)/1000000;
						line += " " + metric + "=" + Math.floor(spMetric*1000000)/1000000;

						double[] combinedScores = new double[11]; 
						for (int i=0; i<11; i++){
							double a = 0.1 * i;
							combinedScores[i] = a*spMetric + (1-a)*irStandardizedScore;
							line += " " + "comb" + i + "=" + Math.floor(combinedScores[i]*1000000)/1000000;
						}

						newLines.add(line);

						System.out.println(line);

						//System.exit(0);
					}
					
					FileHandler.writeListToFile(newLines, outputPath + "/" + f.getName());
				}
			}
			
		}
	}
	
	/**
	 * for sorting map
	 * @param map
	 * @return
	 */
	public static Map<String, Double> sortByComparator(Map<String, Double> map) {
		List<Map.Entry<String, Double>> list = new LinkedList<Map.Entry<String, Double>>(map.entrySet());
		
		Collections.sort(list, new Comparator<Map.Entry<String, Double>>() {
			public int compare(Map.Entry<String, Double> opt1, Map.Entry<String, Double> opt2) {
				return (opt2.getValue().compareTo(opt1.getValue()));
			}
		});
		
		Map<String, Double> sortedMap = new LinkedHashMap<String, Double>();
		
		for (Iterator<Map.Entry<String, Double>> it = list.iterator(); it.hasNext();) {
			Map.Entry<String, Double> entry  = it.next();
			sortedMap.put(entry.getKey(), entry.getValue());
		}
		
		return sortedMap;
	}
	
	public static void printMap(Map<String, Double> map) {
		for (Map.Entry<String, Double> entry : map.entrySet()) {
			System.out.println(entry.getKey() + " : " + entry.getValue());
		}
	}
	
	public static Map<String, Double> extractResults(String inputResultFile, int whichMetric) throws IOException {
		Map<String, Double> ranks = new HashMap<String, Double>();
		
		List<String> lines = FileHandler.readAllLines(inputResultFile);
		
		for (String line : lines) {
			String location = line.split(" ")[0];
			String score = line.split(" ")[whichMetric].split("=")[1].trim();
			double rank = Double.parseDouble(score);
			ranks.put(location, rank);
		}
		
		return sortByComparator(ranks);
	}
	
	/**
	 * extract methods and one of the 11 combined scores, rank
	 * them and save to an output folder
	 * @param inputDir
	 * @param outputDir
	 * @throws IOException
	 */
	public static void extractAndRank(String inputDir, String outputDir) throws IOException{
		File[] files = new File(inputDir).listFiles();
		
		for (int i=0; i<11; i++){
			new File(outputDir + "/" + "comb_" + i).mkdirs();
		}
		
		for (File f : files) {
			if (f.exists() && f.isFile() && ! f.getName().startsWith(".")) {
				for (int i=0; i<11; i++){
					int metricPosition = 8 + i;
					Map<String, Double> rank_i = extractResults(f.getAbsolutePath(), metricPosition);
					List<String> rankedList = mapToRankedList(rank_i);
					String pathToSave = outputDir + "/" + "comb_" + i + "/" + f.getName();
					FileHandler.writeListToFile(rankedList, pathToSave);
					//System.exit(0);
				}
			}
		}
	}
	
	public static void bugLocatorPreprocessing(String file) throws IOException{
		List<String> lines = FileHandler.readAllLines(file);
		List<String> newLines = new ArrayList<String>();
		for (String line : lines) {
			line = line.replace("/", ".");
			//System.out.println(line);
			
			if (line.contains("org.apache")){
				newLines.add("<file>" + "org.apache." + line.split("org.apache.")[1].trim() + "</file>");
			} else {
				System.out.println(line);
			}
		}
		FileHandler.writeListToFile(newLines, file);
	}
	
	public static List<String> mapToRankedList(Map<String, Double> sortedMap) {
		 Iterator<Map.Entry<String, Double>> it = sortedMap.entrySet().iterator();
		 List<String> rankedList = new ArrayList<String>();
		 int rank = 0;
		 while (it.hasNext()) {
			 Map.Entry<String, Double> pair = (Map.Entry<String, Double>)it.next();
			 //System.out.println(pair.getKey() + " = " + pair.getValue());
			 rankedList.add(pair.getKey() + " " + ++rank + " " + pair.getValue());
			 it.remove(); // avoids a ConcurrentModificationException
		 }
		 return rankedList;
	}
	/**
	 * Take coverage raw data of passed and failed tests, and list of items 
	 * (class, method) to be spectrum calculated. And output the final combinied
	 * scores from 0 to 10
	 * @param coverageDir
	 * @param itemsDir
	 * @param outputDir
	 * @throws IOException 
	 */
	public static void fromCoveragePassedFailedToFinalScores(String project, String level) throws IOException {
		String AML_TEST_DIR = "/Users/assist/dmz/src/rtp/DLo/amldata/test_cases_list/" + project;
		String COVERAGE_DIR = "/Users/assist/dmz/gen-data/Spectrum/4SP/sp/" + project; // raw passed and failed trace tests (same for both class and method level)
		String ITEMS_DIR = "/Users/assist/dmz/gen-data/Spectrum/4SP/"; // list of methods or classes to be calculated spectrum
		String SPECTRUM_OUT_DIR = "/Users/assist/dmz/gen-data/Spectrum/4SP/"; // spectrum in the format: class/method ef ep nf np
		
		
		String IR_DIR = "/Users/assist/dmz/gen-data/ir/" + level + "-ir/" + project + "/all/results/pre-fix";// IR folder inputs
		String IRwSP_DIR = "/Users/assist/dmz/gen-data/Spectrum/IRwSP/" + level + "/" + project;
		
		//String IRwSP_METRICS = "/Users/assist/dmz/gen-data/Spectrum/IRwSP_Metrics";
		
		if (level.equals("method")){
			ITEMS_DIR += "method/" + "cmethods/" + project;
			SPECTRUM_OUT_DIR += "method/" + "spectra/" + project;
		} else if (level.equals("class")) {
			ITEMS_DIR += "class/" + "c_classes/" + project;
			SPECTRUM_OUT_DIR += "class/" + "c_spectra/" + project;
		}
		
		if (! new File(SPECTRUM_OUT_DIR).exists()) {
			new File(SPECTRUM_OUT_DIR).mkdir();
		}
		
		computeSPforProject(COVERAGE_DIR, AML_TEST_DIR, ITEMS_DIR, SPECTRUM_OUT_DIR, level); // calculate spectrum information and save it to a folder
		combineSPWithIR(SPECTRUM_OUT_DIR, IR_DIR, IRwSP_DIR);
	}
	
	/**
	 * for ALL projects
	 * @param IRwSP_dir
	 * @param IRwSP_metrics_dir
	 * @throws IOException
	 */
	public static void applyFormulaToIRwSP(String IRwSP_dir, String IRwSP_metrics_dir) throws IOException{
		if (! new File(IRwSP_metrics_dir).exists()) {
			new File(IRwSP_metrics_dir).mkdirs();
		}
		File[] levels = new File(IRwSP_dir).listFiles();
		for (File level : levels) {
			if (level.getName().startsWith(".")) {
				continue;
			}
			File[] projects = new File(level.getAbsolutePath()).listFiles();
			
			for (File project : projects){
				if (project.getName().startsWith(".") || ! project.getName().equals("aspectj")) {
					continue;
				}
				
				new File(IRwSP_metrics_dir + "/" + level.getName() + "/" + project.getName()).mkdir();
				String outputDir = IRwSP_metrics_dir + "/" + level.getName() + "/" + project.getName();
				
				if (project.getName().equals("aspectj")) {
					computeCombinedScoreForAspectj(project.getAbsolutePath(), outputDir);
				} else {
					computeCombinedScore(project.getAbsolutePath(), outputDir);
				}
				
			}
		}
	}
	
	public static void applyFormulaToIRwSP4SingleProject(String IRwSP_dir, String IRwSP_metrics_dir) throws IOException{
		if (! new File(IRwSP_metrics_dir).exists()) {
			new File(IRwSP_metrics_dir).mkdirs();
		}
		
		computeCombinedScore(IRwSP_dir, IRwSP_metrics_dir);
		
//		File[] levels = new File(IRwSP_dir).listFiles();
//		for (File level : levels) {
//			if (level.getName().startsWith(".")) {
//				continue;
//			}
//			File[] projects = new File(level.getAbsolutePath()).listFiles();
//			
//			for (File project : projects){
//				if (project.getName().startsWith(".") || ! project.getName().equals("aspectj")) {
//					continue;
//				}
//				
//				new File(IRwSP_metrics_dir + "/" + level.getName() + "/" + project.getName()).mkdir();
//				String outputDir = IRwSP_metrics_dir + "/" + level.getName() + "/" + project.getName();
//				
//				if (project.getName().equals("aspectj")) {
//					computeCombinedScoreForAspectj(project.getAbsolutePath(), outputDir);
//				} else {
//					computeCombinedScore(project.getAbsolutePath(), outputDir);
//				}
//				
//			}
//		}
	}
	
	public static void applyRankingToMetrics4SingleProject(String inputDir, String outputDir) throws IOException{
		if (! new File(outputDir).exists()) {
			new File(outputDir).mkdirs();
		}
		// traversal through 4 metrics in a given project
		File[] metrics = new File(inputDir).listFiles();
		for (File metric : metrics){
			if (metric.isDirectory() && ! metric.getName().startsWith("."))
			extractAndRank(metric.getAbsolutePath(), outputDir + "/" + metric.getName());
		}
		
//		File[] levels = new File(inputDir).listFiles();
//		for (File level : levels) {
//			if (level.isDirectory() && ! level.getName().startsWith(".") && level.getName().equals("class")) {
//				File[] projects = new File(level.getAbsolutePath()).listFiles();
//				for (File project : projects) {
//					if (project.isDirectory() && ! project.getName().startsWith(".") && project.getName().equals("aspectj")) {
//						File[] metrics = new File(project.getAbsolutePath()).listFiles();
//						for (File metric : metrics){
//							if (metric.isDirectory() && ! metric.getName().startsWith("."))
//							extractAndRank(metric.getAbsolutePath(), outputDir + "/" + level.getName() + "/" + project.getName() + "/" + metric.getName());
//						}
//					}
//					
//				}
//			}
//		}
	}
	
	public static void applyRankingToMetrics(String inputDir, String outputDir) throws IOException{
		if (! new File(outputDir).exists()) {
			new File(outputDir).mkdirs();
		}
		
		File[] levels = new File(inputDir).listFiles();
		for (File level : levels) {
			if (level.isDirectory() && ! level.getName().startsWith(".") && level.getName().equals("class")) {
				File[] projects = new File(level.getAbsolutePath()).listFiles();
				for (File project : projects) {
					if (project.isDirectory() && ! project.getName().startsWith(".") && project.getName().equals("aspectj")) {
						File[] metrics = new File(project.getAbsolutePath()).listFiles();
						for (File metric : metrics){
							if (metric.isDirectory() && ! metric.getName().startsWith("."))
							extractAndRank(metric.getAbsolutePath(), outputDir + "/" + level.getName() + "/" + project.getName() + "/" + metric.getName());
						}
					}
					
				}
			}
		}
	}
	
	
public static void computeFinalResults4SingleProject(String inputDir, String outputDir, String level, String project) throws IOException{
		
		String groundTruthDir0 = "/Users/assist/dmz/src/rtp/DLo/amldata/ground_truth";
		//String groundTruthDir0 = "/Users/assist/dmz/src/rtp/DMZ/ground_truth";
		
		String groundTruthDir = "";
		
		switch (project){
		case "lucene37":
			groundTruthDir = groundTruthDir0 + "/lucene";
			break;
		case "ant53":
			groundTruthDir = groundTruthDir0 + "/ant";
			break;
		case "rhino":
			groundTruthDir = groundTruthDir0 + "/rhino";
			break;
		case "aspectj":
			groundTruthDir = groundTruthDir0 + "/aspectj";
			break;
		}
		
		if (! new File(outputDir).exists()) {
			new File(outputDir).mkdirs();
		}
		
		File[] metrics = new File(inputDir).listFiles();
		for (File metric : metrics){
			if (metric.isDirectory() && ! metric.getName().startsWith(".")) {
				File[] combs = new File(metric.getAbsolutePath()).listFiles();
				for (File comb : combs) {
					if (comb.isDirectory() && ! comb.getName().startsWith(".")) {
						String relOutDir = outputDir + "/" + metric.getName() ;
						new File(relOutDir).mkdirs();
						relOutDir += "/" + comb.getName();
						
						if (comb.isDirectory() && ! comb.getName().startsWith(".")) {
							if (level.equals("class")) {
								MetricsCalculator4IR.calProjectMetrics(comb.getAbsolutePath(), groundTruthDir, relOutDir);
							} else if (level.equals("method")) {
								MetricsCalculator4IRMethod.calProjectMetrics(comb.getAbsolutePath(), groundTruthDir, relOutDir);
							}
						}	
					}
													
				}
			}
		}
		
//		File[] levels = new File(inputDir).listFiles();
//		for (File level : levels) {
//			if (level.isDirectory() && ! level.getName().startsWith(".") && level.getName().equals("class")) {
//				File[] projects = new File(level.getAbsolutePath()).listFiles();
//				for (File project : projects){
//					if (project.isDirectory() && ! project.getName().startsWith(".") && project.getName().equals("aspectj")){
//						String groundTruthDir = "";
//						
//						switch (project.getName()){
//						case "lucene37":
//							groundTruthDir = groundTruthDir0 + "/lucene";
//							break;
//						case "ant53":
//							groundTruthDir = groundTruthDir0 + "/ant";
//							break;
//						case "rhino":
//							groundTruthDir = groundTruthDir0 + "/rhino";
//							break;
//						case "aspectj":
//							groundTruthDir = groundTruthDir0 + "/aspectj";
//							break;
//						}
//						
//						File[] metrics = new File(project.getAbsolutePath()).listFiles();
//						for (File metric : metrics){
//							if (metric.isDirectory() && ! metric.getName().startsWith(".")) {
//								File[] combs = new File(metric.getAbsolutePath()).listFiles();
//								for (File comb : combs) {
//									if (comb.isDirectory() && ! comb.getName().startsWith(".")) {
//										String relOutDir = outputDir + "/" + level.getName() + "/" + project.getName() + "/" + metric.getName() ;
//										new File(relOutDir).mkdirs();
//										relOutDir += "/" + comb.getName();
//										
//										if (comb.isDirectory() && ! comb.getName().startsWith(".")) {
//											if (level.getName().equals("class")) {
//												MetricsCalculator4IR.calProjectMetrics(comb.getAbsolutePath(), groundTruthDir, relOutDir);
//											} else if (level.getName().equals("method")) {
//												MetricsCalculator4IRMethod.calProjectMetrics(comb.getAbsolutePath(), groundTruthDir, relOutDir);
//											}
//										}	
//									}
//																	
//								}
//							}
//						}
//					}
//				}
//			}
//		}

	}
	
	public static void computeFinalResults(String inputDir, String outputDir) throws IOException{
		
		//String groundTruthDir0 = "/Users/assist/dmz/src/rtp/DLo/amldata/ground_truth";
		
		String groundTruthDir0 = "/Users/assist/dmz/src/rtp/DMZ/ground_truth";
		
		if (! new File(outputDir).exists()) {
			new File(outputDir).mkdirs();
		}
		
		File[] levels = new File(inputDir).listFiles();
		for (File level : levels) {
			if (level.isDirectory() && ! level.getName().startsWith(".") && level.getName().equals("class")) {
				File[] projects = new File(level.getAbsolutePath()).listFiles();
				for (File project : projects){
					if (project.isDirectory() && ! project.getName().startsWith(".") && project.getName().equals("aspectj")){
						String groundTruthDir = "";
						
						switch (project.getName()){
						case "lucene37":
							groundTruthDir = groundTruthDir0 + "/lucene";
							break;
						case "ant53":
							groundTruthDir = groundTruthDir0 + "/ant";
							break;
						case "rhino":
							groundTruthDir = groundTruthDir0 + "/rhino";
							break;
						case "aspectj":
							groundTruthDir = groundTruthDir0 + "/aspectj";
							break;
						}
						
						File[] metrics = new File(project.getAbsolutePath()).listFiles();
						for (File metric : metrics){
							if (metric.isDirectory() && ! metric.getName().startsWith(".")) {
								File[] combs = new File(metric.getAbsolutePath()).listFiles();
								for (File comb : combs) {
									if (comb.isDirectory() && ! comb.getName().startsWith(".")) {
										String relOutDir = outputDir + "/" + level.getName() + "/" + project.getName() + "/" + metric.getName() ;
										new File(relOutDir).mkdirs();
										relOutDir += "/" + comb.getName();
										
										if (comb.isDirectory() && ! comb.getName().startsWith(".")) {
											if (level.getName().equals("class")) {
												MetricsCalculator4IR.calProjectMetrics(comb.getAbsolutePath(), groundTruthDir, relOutDir);
											} else if (level.getName().equals("method")) {
												MetricsCalculator4IRMethod.calProjectMetrics(comb.getAbsolutePath(), groundTruthDir, relOutDir);
											}
										}	
									}
																	
								}
							}
						}
					}
				}
			}
		}

	}
	
	/**
	 * Read results to format a table
	 * @param dirResults
	 * @throws IOException 
	 */
	public static void readResults (String resultDir, String outputDir) throws IOException {
		File[] levels = new File(resultDir).listFiles();
		
		for (File level : levels) {
			if (level.getName().startsWith(".")) {
				continue;
			}
			
			System.out.println("=======================" + level.getName() + "============================ \n");
			
			File[] projects = new File(level.getAbsolutePath()).listFiles();
			
			for (File project : projects) {
				if (project.getName().startsWith(".")) {
					continue;
				}
				
				
				File[] metrics = new File(project.getAbsolutePath()).listFiles();
				
				for (File metric : metrics) {
					if (metric.getName().startsWith(".")){
						continue;
					}
					
//					File[] combs = new File(metric.getAbsolutePath()).listFiles();
					
					List<List<String>> combsList = new ArrayList<List<String>>();
					
					for (int k=0; k<11; k++) {
//						if (comb.getName().startsWith(".")) {
//							continue;
//						}
						
						String comb = metric.getAbsolutePath() + "/" + "comb_" + k;
						
						List<String> lines = FileHandler.readAllLines(comb);
						
						String topLine = " " , metricLine = "";
						
						for (int i = lines.size()-1; i>0; i--) {
							if (lines.get(i).startsWith("Top")) {
								topLine = lines.get(i);
							}
							
							if (lines.get(i).startsWith("MRR")) {
								metricLine = lines.get(i);
							}
							
							if (lines.get(i).contains("Overall Metrics")) {
								break;
							}
						}
						
						List<String> combResults = new ArrayList<String>();
						
						//combResults.add("comb_" + k);
						//int index = Integer.parseInt(comb.getName().split("_")[1]);
						combResults.add(topLine.split(";")[0].split("=")[1].trim()); //top1
						combResults.add(topLine.split(";")[1].split("=")[1].trim()); //top5
						combResults.add(topLine.split(";")[2].split("=")[1].trim()); //top10
						combResults.add(metricLine.split(";")[0].split("=")[1].trim()); //mrr
						combResults.add(metricLine.split(";")[1].split("=")[1].trim());//map
						
						combsList.add(k, combResults);
					}
					
					List<String> lines = new ArrayList<String>();
//					for (int i=0; i < combsList.get(0).size(); i++){
//						lines.add("");
//					}
//					
//					for (int i = 0; i < combsList.size(); i++) {
//						for (int j = 0; j<combsList.get(i).size(); j++) {
//							lines.add(j, lines.get(j) + " " + combsList.get(i).get(j));
//						}
//					}
					
					String tableCaption = String.format("%9s %9s %7s %6s %6s %6s %6s %6s %6s %6s %6s %6s %6s %6s","Project", "Formula", "Metric|", "comb0", "comb1","comb2","comb3","comb4","comb5","comb6","comb7","comb8","comb9","comb10");
					System.out.println(tableCaption);		
					for (int j=0; j< 5 ; j ++) {
						String line = "";
						for (int i=0; i<11; i++) {
							if (j>2) {
								String value = String.format("%.4f ", Double.parseDouble(combsList.get(i).get(j)));
								line +=  value;
							} else {
								line += String.format("% 6d ", Integer.parseInt(combsList.get(i).get(j))) ;
							}
						}
						String rowTitle = "";
						if (j==0){
							rowTitle = String.format("%7s", "Top_1|"); // + line;
						} else if (j == 1) {
							rowTitle = String.format("%7s", "Top_5|"); // + line;
						} else if (j == 2) {
							rowTitle = String.format("%7s", "Top_10|"); // + line;
							
						} else if (j== 3) {
							rowTitle = String.format("%7s", "MRR|"); // + line;
							
						} else if (j ==4) {
							rowTitle = String.format("%7s", "MAP|"); // + line;
							
						}
						
						rowTitle = String.format("%9s", project.getName()) + " " + String.format("%9s", metric.getName()) + " " + rowTitle;
						
						rowTitle = String.format("%27s", rowTitle);
						
//						System.out.println(project.getName() + " " + metric.getName() + " " + line);
						System.out.println(rowTitle + " " + line);
					}
					
					System.out.println("\n");
					
//					for (String line : lines) {
//						System.out.println(line);
//					}
					
				}
				
			}
			
		}
	}
	/**
	 * Given IR and SP data, compute to the final results
	 * Via different steps
	 * @param irInputDir
	 * @param spInputDir
	 * @param combinedOutputDir
	 * @throws IOException
	 */
	public static void combineIRandSP_ALL(String irInputDir, String spInputDir, String combinedOutputDir) throws IOException {
		
		if (! new File(combinedOutputDir).exists()) {
			new File(combinedOutputDir).mkdirs();
		}
		
		// traversal ir dir first
		File[] irMethods = new File(irInputDir).listFiles(); 
		// methods are bluir, ir, buglocator
		for (File irMethod : irMethods) { 
			if (irMethod.getName().startsWith(".") || ! irMethod.getName().equals("buglocator")) { // || ! irMethod.getName().startsWith("ir")
				continue;
			}
			
			// then traversal the levels: class and method
			File[] levels = new File(irMethod.getAbsolutePath()).listFiles();
			
			for (File level : levels) {
				if (level.getName().startsWith(".")) {
					continue;
				}
				//then traversal the projects: aspectj, ant53, lucnene37, and rhino
				File[] projects = new File(level.getAbsolutePath()).listFiles();
				for (File project : projects) {
					if (project.getName().startsWith(".")) {
						continue;
					}
					
					String inputIRPath = project.getAbsolutePath();
					String inputSPPath = spInputDir + "/" + level.getName() + "/" + project.getName();
					String combinedIRwSPPath = combinedOutputDir + "/" + irMethod.getName() + "/" + "IRwSP" + "/" + level.getName() + "/" + project.getName();
					combineSPWithIR(inputIRPath, inputSPPath, combinedIRwSPPath);
					
					//apply formula to IRwSP and save results to IRwSP_Metrics for each project
					String IRwSP_Metrics_dir = combinedOutputDir + "/" + irMethod.getName() + "/" + "IRwSP_Metrics" + "/" + level.getName() + "/" + project.getName();
					applyFormulaToIRwSP4SingleProject(combinedIRwSPPath, IRwSP_Metrics_dir);
					
					//rank the results based on the score combi
					String IRwSP_Ranks = combinedOutputDir + "/" + irMethod.getName() + "/" + "IRwSP_Ranks" + "/" + level.getName() + "/" + project.getName();
					applyRankingToMetrics4SingleProject(IRwSP_Metrics_dir, IRwSP_Ranks);
					
					String IRwSP_Results = combinedOutputDir + "/" + irMethod.getName() + "/" + "IRwSP_Results" + "/" + level.getName() + "/" + project.getName();
					computeFinalResults4SingleProject(IRwSP_Ranks, IRwSP_Results, level.getName(), project.getName());
				}
			}
		}
	}
	
	public static void main(String[] args) throws IOException {
		
		
		//String spDir = "/Users/assist/dmz/gen-data/Spectrum/4SP/sp/ant53";
		String spDir = "/Users/assist/IR-SP-combined/sp-data/aspectj";
		
		String amlTestDir = "/Users/assist/dmz/src/rtp/DLo/amldata/test_cases_list/aspectj";
		
		//String itemDir = "/Users/assist/dmz/gen-data/Spectrum/4SP/class/c_classes/ant53";
		String itemDir = "/Users/assist/dmz/gen-data/Spectrum/4SP/class/c_classes/aspectj";
		
		//String itemSpectrumOutputDir = "/Users/assist/dmz/gen-data/Spectrum/4SP/class/c_spectra/ant53";
		String itemSpectrumOutputDir = "/Users/assist/dmz/gen-data/Spectrum/4SP/class/c_spectra/aspectj";
		
		String level = "class";
		
		//methodNameCorrection("/Users/assist/dmz/gen-data/4SP/classes/rhino", "/Users/assist/dmz/gen-data/4SP/c_classes/rhino");
		
		//computeSPforProject (spDir, amlTestDir, itemDir, itemSpectrumOutputDir, level);
		
		//computeSPforProject("/Users/assist/IR-SP-combined/sp-data/aspectj", amlTestDir, "/Users/assist/IR-SP-combined/ir-data/ir/class/aspectj", "/Users/assist/IR-SP-combined/ir-sp-scores/ir/class/aspectj", level);
		
		//combineSPWithIR("/Users/assist/dmz/gen-data/Spectrum/4SP/method/spectra/ant53", "/Users/assist/dmz/gen-data/ir/method-ir/ant53/all/results/pre-fix", "/Users/assist/dmz/gen-data/Spectrum/IRwSP/method/ant53");
		
		//computeCombinedScore("/Users/assist/dmz/gen-data/IRwSP/class/rhino", "/Users/assist/dmz/gen-data/IRwSP_Tarantula/class/rhino");
		
		//extractAndRank("/Users/assist/dmz/gen-data/Spectrum/IRwSP_Tarantula/method/rhino", "/Users/assist/dmz/gen-data/Spectrum/IRwSP_Tarantula_Variations/method/projects/rhino");
		
		//bugLocatorPreprocessing("/Users/assist/Desktop/lucene-2717-files.txt");
		
		//applyFormulaToIRwSP("/Users/assist/dmz/gen-data/Spectrum/ir/IRwSP", "/Users/assist/dmz/gen-data/Spectrum/ir/IRwSP_Metrics");
		
		//applyRankingToMetrics("/Users/assist/dmz/gen-data/Spectrum/ir/IRwSP_Metrics", "/Users/assist/dmz/gen-data/Spectrum/ir/IRwSP_Ranks");
		
		//computeFinalResults("/Users/assist/dmz/gen-data/Spectrum/ir/IRwSP_Ranks", "/Users/assist/dmz/gen-data/Spectrum/ir/IRwSP_Results");
		
		//methodNameCorrection("/Users/assist/dmz/gen-data/Spectrum/4SP/method/extracted_methods/ant53", "/Users/assist/dmz/gen-data/Spectrum/4SP/method/cmethods/ant53");
		
		//readResults ("/Users/assist/Desktop/IRSPcombined/combined/ir-cv-sp/ir/IRwSP_Results", "");
		
		//readResults (args[0], "");
		//combineIRandSP_ALL("/Users/assist/Desktop/IRSPcombined/ir-cv-data", "/Users/assist/Desktop/IRSPcombined/sp-data", "/Users/assist/Desktop/IRSPcombined/combined/ir-cv-sp");
		
//		System.out.println("Done working with IR+CV");
//		
//		combineIRandSP_ALL("/Users/assist/Desktop/IRSPcombined/ir-data", "/Users/assist/Desktop/IRSPcombined/sp-data", "/Users/assist/Desktop/IRSPcombined/combined/ir-only-sp");
		
		//combineIRandSP_ALL("/Users/assist/Desktop/IRSPcombined/ir-slicing-data", "/Users/assist/Desktop/IRSPcombined/sp-data", "/Users/assist/Desktop/IRSPcombined/combined/ir-slicing-sp");
		
		readResults ("/Users/assist/Desktop/IRSPcombined/combined/ir-slicing-sp/buglocator/IRwSP_Results", "");
			
	}
		

}
