package td.utils;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class SpectrumBasedApproach {
	
	public static String amlTestDir = "/Users/assist/dmz/src/rtp/DLo/amldata/test_cases_list";
//	public static String baseIRSPDir = "/Users/assist/dmz/IR-SP";
	
	public static String baseIRSPDir = "/Users/assist/IR-SP-combined/ir-sp-scores";
	
//	public static String project = "";
//	
//	public static String irClassDir = "/Users/assist/dmz/gen-data/ir/class-ir/" + project + "/all/results/pre-fix";
//	public static String irMethodDir = "/Users/assist/dmz/gen-data/ir/method-ir/" + project + "/all/results/pre-fix";
//	
//	public static String bluirClassDir = "/Users/assist/dmz/gen-data/sIR/out/" + project ;
//	public static String bluirMethodDir = "/Users/assist/dmz/gen-data/sIR_method/out/" + project; 
//	
//	public static String buglocatorClassDir = "";
//	public static String buglocatorMethodDir = "";
	
	public static String spDir = "/Users/assist/IR-SP-combined/sp-data";
	
	public static String getIrProjectDir (String project, String irMethod, String level) {
		if (irMethod.equals("ir") && level.equals("class")){
			String irClassDir = "/Users/assist/dmz/gen-data/ir/class-ir/" + project + "/all/results/pre-fix";
			return irClassDir;
		}
		
		if (irMethod.equals("ir") && level.equals("method")){
			String irMethodDir = "/Users/assist/dmz/gen-data/ir/method-ir/" + project + "/all/results/pre-fix";
			return irMethodDir;
		}
		
		if (irMethod.equals("bluir") && level.equals("class")){
			String bluirClassDir = "/Users/assist/dmz/gen-data/sIR/out/" + project ;
			return bluirClassDir;
		}
		
		if (irMethod.equals("bluir") && level.equals("method")){
			String bluirMethodDir = "/Users/assist/dmz/gen-data/sIR_method/out/" + project; 
			return bluirMethodDir;
		}
		
		return null;
		
	}
	
	public static String getAmlTestListFilePath(String project, String id) {
		return amlTestDir + "/" + project + "/" + id;
	}
	
	/**
	 * given:
	 * 	- methodCoverage of all tests: passed and failed in faulttracer format; identified by a unique bug ID
	 * 	- test list given by original AML that tells which test is failed and passed
	 * return: a TestsExecution class contains 
	 * 	- a list of testcase, each test case contains: test name, result, list of covered methods
	 *  - number of failed tests
	 *  - number of passed tests
	 * 	- total number of all tests
	 * @param coverageFilePath
	 * @param amlTestFilePath
	 * @return
	 * @throws IOException
	 */
	public static TestsExecution extractTestsExecution(String coverageFilePath, String amlTestFilePath) throws IOException {
		return new TestsExecution(coverageFilePath, amlTestFilePath);
	}
	
	public static void extractIRRankedLists(String inputBaseDir, String outputBaseDir, String rankingMethod, String level, String project) throws IOException{
		String outputDir = outputBaseDir + "/" + rankingMethod + "/" + level + "/" + project;
		new File(outputDir).mkdirs();
		
		File[] files = new File(inputBaseDir).listFiles();
		
		for (File file : files) {
			if (! file.getName().startsWith(".")) {
				String inputFile = file.getAbsolutePath();
				String outputFile = outputDir + "/" + file.getName();
				List<String> rankedIRList = extractItemsFromIRbasedRankedList(inputFile, rankingMethod);
				FileHandler.writeListToFile(rankedIRList, outputFile);
			}
		}
	}
	
	public static List<String> extractItemsFromIRbasedRankedList(String filePathToList, String rankingMethod){ // rankingmethod = IR | BLUiR | BL
		List<String> rankedList = new ArrayList<String>();
		
		try {
			rankedList = FileHandler.readAllLines(filePathToList);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println("Reading IR result list wrong at: " + filePathToList);
		}
		
		List<String> extractedList = new ArrayList<String>();
		int itemNamePos = 0, itemScorePos = 0;
		String dem = "";
		String irPrefix = "";
		switch (rankingMethod){
		case "ir":
			itemNamePos = 2;
			itemScorePos = 4;
			dem = " "; 
			irPrefix = "ir";
			break;
		case "bluir":
			itemNamePos = 2;
			itemScorePos = 4;
			dem = " ";
			irPrefix = "bluir";
			break;
		case "buglocator":
			itemNamePos = 1;
			itemScorePos = 3;
			dem = ",";
			irPrefix = "bl";
			break;
			default:
				System.out.println("[Error]. wrong ranking method");
				System.exit(0);
		}
		
		for (String item : rankedList){
			String itemName = item.split(dem)[itemNamePos];
			String itemScore = item.split(dem)[itemScorePos];
			String extractedItem = itemName + " " + irPrefix + "=" + itemScore;
			System.out.println(extractedItem);
			extractedList.add(extractedItem);
		}
		
		return extractedList;
	}
	
	public static void combineSPwithIR(String project, String irMethod, String level) throws IOException {
		String spProjectDir = spDir + "/" + project;
		String irProjectDir = getIrProjectDir(project, irMethod, level);
		
		//String outputPath = baseIRSPDir + "/" + project + "/" + irMethod + "/" + level;
		
		String outputPath = baseIRSPDir + "/" + irMethod + "/" + level + "/" + project;
		
		File[] spFiles = new File(spProjectDir).listFiles();
		for (File file : spFiles) {
			
			if (file.getName().startsWith(".")) {
				continue;
			}
			
			if (new File (outputPath + "/" + file.getName()).exists()) {
				continue;
			}
			
			String amlTestListFilePath = getAmlTestListFilePath(project, file.getName());
			String irProjectIDFilePath = irProjectDir + "/" + file.getName();
			
			if (! new File(irProjectIDFilePath).exists()) {
				if (file.getName().endsWith(".txt")) {
					irProjectIDFilePath = irProjectDir + "/" + file.getName().split("\\.")[0];
				} else {
					irProjectIDFilePath = irProjectDir + "/" + file.getName() + ".txt";
				}
			}
			
			//List<String> irRankedList = extractItemsFromIRbasedRankedList(irProjectIDFilePath, irMethod);
			
			TestsExecution te;
			
			if (project.equals("aspectj")){
				te = new TestsExecution(file.getAbsolutePath());
			} else {
				te = new TestsExecution(file.getAbsolutePath(), amlTestListFilePath);
			}
			
			//List<String> combinedList = te.getSpectrumForListOfItems(irRankedList, level);
			
			List<String> combinedList = te.getSpectrumForListOfItems2(irProjectIDFilePath, irMethod, level);
			
			
			new File(outputPath).mkdirs();
			
			FileHandler.writeListToFile(combinedList, outputPath + "/" + file.getName());
		}
	}
	
	public static void computeCombinedScore(String inputDir, String outputDir) throws IOException {
		File[] files = new File(inputDir).listFiles();
		
		String[] metrics = {"T", "J", "A", "O"};
		
		for (String metric : metrics) {
			
			String outputPath = "";

			switch (metric){
			case "T":
				outputPath = outputDir + "/" + "tarantula/";
				break;
			case "J":
				outputPath = outputDir + "/" + "jaccard/" ;
				break;
			case "A":
				outputPath = outputDir + "/" + "ample/";
				break;
			case "O":
				outputPath = outputDir + "/" + "ochiai/";
				break;
			default:
				System.out.println("invalid metric value");
				System.exit(0);
			}
			
			new File(outputPath).mkdirs();
			
			for (File f : files) {
				if (f.exists() && f.isFile() && ! f.getName().startsWith(".")){
					List<String> lines = FileHandler.readAllLines(f.getAbsolutePath());

					List<String> newLines = new ArrayList<String>();

					double irMin = 100000000, irMax = 0;
					for (String line : lines) {
						double irScore = Double.parseDouble(line.split(" ")[1].split("=")[1]);
						if (irScore>irMax) {
							irMax = irScore;
						}
						if (irMin > irScore) {
							irMin = irScore;
						}
					}

					for (String line : lines) {
						double irScore = Double.parseDouble(line.split(" ")[1].split("=")[1]);
						double irStandardizedScore = (irScore - irMin)/(irMax-irMin);
						String ef = line.split(" ")[2].split("=")[1];
						String ep = line.split(" ")[3].split("=")[1];
						String nf = line.split(" ")[4].split("=")[1];
						String np = line.split(" ")[5].split("=")[1];

						double spMetric = SPFormula.getMetric(metric, ef, ep, nf, np);
						
						if (spMetric < 0) {
							System.out.println("Metric value is incorrect");
							System.exit(0);
						}

						line += " " + "IR=" + Math.floor(irStandardizedScore*1000000)/1000000;
						line += " " + metric + "=" + Math.floor(spMetric*1000000)/1000000;

						double[] combinedScores = new double[11]; 
						for (int i=0; i<11; i++){
							double a = 0.1 * i;
							combinedScores[i] = a*spMetric + (1-a)*irStandardizedScore;
							line += " " + "comb" + i + "=" + Math.floor(combinedScores[i]*1000000)/1000000;
						}

						newLines.add(line);

						System.out.println(line);

						//System.exit(0);
					}
					
					FileHandler.writeListToFile(newLines, outputPath + "/" + f.getName());
				}
			}
			
		}
	}
	/**
	 * performe computation on the dir IR-SP to create IR-SP-COMS
	 * @param inputDir
	 * @param outputDir
	 * @throws IOException
	 */
	public static void applyFormulaAndCombs(String inputDir, String outputDir) throws IOException{
		File[] projects = new File(inputDir).listFiles();
		for (File project : projects) {
			if (project.getName().startsWith(".")) {
				continue;
			}
			
			File[] irMethods = new File(project.getAbsolutePath()).listFiles();
			
			for (File method : irMethods) {
				if (method.getName().startsWith(".")){
					continue;
				}
				
				File[] levels = new File(method.getAbsolutePath()).listFiles();
				for (File level : levels) {
					if (level.getName().startsWith(".")) {
						continue;
					}
					String outputPath = outputDir + "/" + project + "/" + method + "/" + level;
					new File(outputPath).mkdirs();
					computeCombinedScore(level.getAbsolutePath(),outputPath);
				}
			}
		}
	}
	
	public static void doRankingOnMetrics(String inputDir, String outputDir) throws IOException {
		File[] projects = new File(inputDir).listFiles();
		for (File project : projects){
			if (project.getName().startsWith(".")){
				continue;
			}
			
			File[] irMethods = new File(project.getAbsolutePath()).listFiles();
			
			for (File method : irMethods) {
				if (method.getName().startsWith(".")) {
					continue;
				}
				
				File[] levels = new File(method.getAbsolutePath()).listFiles();
				
				for (File level : levels){
					if (level.getName().startsWith(".")){
						continue;
					}
					
					File[] metrics = new File(level.getAbsolutePath()).listFiles();
					
					for (File metric : metrics) {
						if (metric.getName().startsWith(".")){
							continue;
						}
						String outputPath = outputDir + "/" + project + "/" + method + "/" + metric;
						extractAndRank(metric.getAbsolutePath(), outputPath);
						
					}
					
				}
			}
		}
	}
	
	/**
	 * extract methods and one of the 11 combined scores, rank
	 * them and save to an output folder
	 * @param inputDir
	 * @param outputDir
	 * @throws IOException
	 */
	public static void extractAndRank(String inputDir, String outputDir) throws IOException{
		File[] files = new File(inputDir).listFiles();
		
		for (int i=0; i<11; i++){
			new File(outputDir + "/" + "comb_" + i).mkdirs();
		}
		
		for (File f : files) {
			if (f.exists() && f.isFile() && ! f.getName().startsWith(".")) {
				for (int i=0; i<11; i++){
					int metricPosition = 8 + i;
					Map<String, Double> rank_i = extractResults(f.getAbsolutePath(), metricPosition);
					List<String> rankedList = mapToRankedList(rank_i);
					String pathToSave = outputDir + "/" + "comb_" + i + "/" + f.getName();
					FileHandler.writeListToFile(rankedList, pathToSave);
					//System.exit(0);
				}
			}
		}
	}
	
	public static List<String> mapToRankedList(Map<String, Double> sortedMap) {
		 Iterator<Map.Entry<String, Double>> it = sortedMap.entrySet().iterator();
		 List<String> rankedList = new ArrayList<String>();
		 int rank = 0;
		 while (it.hasNext()) {
			 Map.Entry<String, Double> pair = (Map.Entry<String, Double>)it.next();
			 //System.out.println(pair.getKey() + " = " + pair.getValue());
			 rankedList.add(pair.getKey() + " " + ++rank + " " + pair.getValue());
			 it.remove(); // avoids a ConcurrentModificationException
		 }
		 return rankedList;
	}
	
	public static Map<String, Double> extractResults(String inputResultFile, int whichMetric) throws IOException {
		Map<String, Double> ranks = new HashMap<String, Double>();
		
		List<String> lines = FileHandler.readAllLines(inputResultFile);
		
		for (String line : lines) {
			String location = line.split(" ")[0];
			String score = line.split(" ")[whichMetric].split("=")[1].trim();
			double rank = Double.parseDouble(score);
			ranks.put(location, rank);
		}
		
		return sortByComparator(ranks);
	}
	
	/**
	 * for sorting map
	 * @param map
	 * @return
	 */
	public static Map<String, Double> sortByComparator(Map<String, Double> map) {
		List<Map.Entry<String, Double>> list = new LinkedList<Map.Entry<String, Double>>(map.entrySet());
		
		Collections.sort(list, new Comparator<Map.Entry<String, Double>>() {
			public int compare(Map.Entry<String, Double> opt1, Map.Entry<String, Double> opt2) {
				return (opt2.getValue().compareTo(opt1.getValue()));
			}
		});
		
		Map<String, Double> sortedMap = new LinkedHashMap<String, Double>();
		
		for (Iterator<Map.Entry<String, Double>> it = list.iterator(); it.hasNext();) {
			Map.Entry<String, Double> entry  = it.next();
			sortedMap.put(entry.getKey(), entry.getValue());
		}
		
		return sortedMap;
	}
	/**
	 * combine spectra with ir score to create IRwSP files
	 * @param irDir
	 * @param spDir
	 * @param combinedDir
	 * @throws IOException
	 */
	public static void computeIRwSP(String irDir, String spDir, String combinedDir) throws IOException {
		File[] spFiles = new File(spDir).listFiles();
		for (File spFile : spFiles) {
			if (spFile.getName().startsWith(".")) {
				continue;
			}
			
			List<String> spLines = FileHandler.readAllLines(spFile.getAbsolutePath());
			List<String> irLines = FileHandler.readAllLines(irDir + "/" + spFile.getName());
			List<String> combinedLines = new ArrayList<String>();
			
			for (String spLine : spLines) {
				
				String spName = spLine.split(" ")[0];
				if (! spName.endsWith(".java")) {
					spName = spName + ".java";
				}
				
				for (String irLine : irLines) {
					String irName = irLine.split(" ")[2];
					if (! irName.endsWith(".java")) {
						irName = irName + ".java";
					}
					
					if (spName.endsWith(irName) || irName.endsWith(spName)) {
						String irScore = irLine.split(" ")[4].trim();
						combinedLines.add(spLine + " " + "bluir=" + irScore);
						break;
					}
				}
			}
			
			FileHandler.writeListToFile(combinedLines, combinedDir + "/" + spFile.getName());
		}
	}
	
	public static void main(String[] args) throws IOException {
		String project = "aspectj";
		String irMethod = "ir";
		String level = "method";
		
		combineSPwithIR(project, irMethod, level);
		//combineSPwithIR(project, irMethod, "method");
		
//		String outputBaseDir = "/Users/assist/dmz/IR-ranked-lists";
//		String rankingMethod = "bluir";
//		String level = "method";
//		String project = "aspectj";
//		
//		String inputBaseDir = "/Users/assist/dmz/gen-data/sIR_method/out" + "/" + project;
//		
//		extractIRRankedLists(inputBaseDir, outputBaseDir, rankingMethod, level, project);
	}

}
