package td.utils;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class SpectrumProcessor {
	
	public static Set<String> mcvToClasses(String coverageFile) {
		List<String> lines = new ArrayList<String>();
		try {
			lines = FileHandler.readAllLines(coverageFile);
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println(coverageFile);
		}
		
		ArrayList<String> classes = new ArrayList<String>();
		
		for (String line : lines) {
			String className = "";
			boolean found = false;
			for (String s : line.split("\\.")) {
				if (Character.isUpperCase(s.charAt(0))) {
					className += s;
					found = true;
					break;
				} else {
					className += s + ".";
				}
			}
			
			if (! found) {
				continue;
			}
			
			if (className.contains("$")) {
				className = className.split("\\$")[0];
			}
			
			classes.add(className.replace(".", "/") + ".java");
		}
		
		HashSet<String> uniqueClasses = new HashSet<>(classes);
		
		return uniqueClasses;
	}
	/**
	 * raw slicing data to method slicing format 
	 * @param sliceFile
	 * @return
	 */
	public static Set<String> sliceToMethods(String sliceFile){
		List<String> lines = new ArrayList<String>();
		try {
			lines = FileHandler.readAllLines(sliceFile);
		} catch (Exception e){
			e.printStackTrace();
			System.out.println("[Error] while reading file: " + sliceFile);
		}
		
		ArrayList<String> methods = new ArrayList<String>();

		for (String line : lines) {
			if (line.contains("The dynamic slice for criterion") || line.contains("Slice consists") || line.contains("Computation took")) {
				continue;
			} else {
				methods.add(line.split(":")[0]);
			}
		}
		
		HashSet<String> uniqueMethods = new HashSet<>(methods);
		
		return uniqueMethods;
		
	}
	
	/**
	 * raw slicing data to class slices
	 * @param sliceFile
	 * @return
	 */
	public static Set<String> sliceToClasses(String sliceFile) {
		List<String> lines = new ArrayList<String>();
		try {
			lines = FileHandler.readAllLines(sliceFile);
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println(sliceFile);
		}
		
//		lines.remove(0); // remove the header line
//		lines.remove(lines.size()-1); // remove the information line
//		lines.remove(lines.size()-1); //remove the information line
		
//		for (String line : lines) {
//			if (line.contains("The dynamic slice for criterion") || line.contains("Slice consists") || line.contains("Computation took")) {
//				lines.remove(line);
//			}
//		}
		
		ArrayList<String> classes = new ArrayList<String>();
		
		for (String line : lines) {
			
			if (line.contains("The dynamic slice for criterion") || line.contains("Slice consists") || line.contains("Computation took")) {
				continue;
			}
			
			String className = "";
			boolean found = false;
			for (String s : line.split("\\.")) {
				if (Character.isUpperCase(s.charAt(0)) || s.startsWith("_")) {
					className += s;
					found = true;
					break;
				} else {
					className += s + ".";
				}
			}
			
			if (! found) {
				System.out.println("[warning] class name is not found in a line : ");
				System.out.println(sliceFile + "=>" + line);
				continue;
			}
			
			if (className.contains("$")) {
				className = className.split("\\$")[0];
			}
			
			// classes.add(className.replace(".", "/") + ".java"); //full name
			classes.add(className.split("\\.")[className.split("\\.").length-1] + ".java");
		}
		
		HashSet<String> uniqueClasses = new HashSet<>(classes);
		
		return uniqueClasses;
		
	}
	
	
	public static void printStringList(Set<String> list) {
		for (String s : list) {
			System.out.println(s);
		}
	}
	
	public static void writeToFile(Set<String> lines, String filePath) {
		try {
			FileWriter fw = new FileWriter(filePath);
			for (String line : lines) {
				fw.write(line + System.getProperty("line.separator"));
				
			}
			fw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println(filePath);
		}
		
	}
	
	public static void batchProcess(String sourceDir, String destDir, String type, String level) {
		File[] files = new File(sourceDir).listFiles();
		for (File file : files) {
			String name = file.getName().split("\\.")[0];
			String sourcePath = file.getAbsolutePath();
			String destPath = destDir + "/" + name;
			Set<String> lines = new HashSet<String>();
			if (type.equals("coverage")) {
				lines = mcvToClasses(sourcePath);
			} else if (type.equals("slicing") && level.equals("class")) {
				lines = sliceToClasses(sourcePath);
			} else if (type.equals("slicing") && level.equals("method")) {
				lines = sliceToMethods(sourcePath);
			}
			
			writeToFile(lines, destPath);
		}
	}
	
	public static void copyFile(String src, String dest) throws IOException{
		String[] cmd = {"rsync", "-avR", src, dest};
		Runtime.getRuntime().exec(new String[] {"cd", "/Users/assist/dmz/gen-data/faulttracer"});
		Runtime.getRuntime().exec(cmd);
	}

	public static void main(String[] args) throws IOException {
		// TODO Auto-generated method stub
//		String destDir = "/Users/assist/dmz/filters/coverage/rhino";
//		String spectrumDir = "/Users/assist/dmz/gen-data/faulttracer/rhino/post-fix/coverage-data/union/method";
//		batchProcess(spectrumDir, destDir, "coverage");
		
		String[] projects = new String[]{"lucene37"};

		for (String p : projects) {
			String destDir = "/Users/assist/dmz/filters/slicing/method/" + p;
			File f = new File(destDir);
			if (! f.exists()) {
				f.mkdir();
			}
			String spectrumDir = "/Users/assist/dmz/gen-data/jslicer/slices/" + p + "/union";
			batchProcess(spectrumDir, destDir, "slicing", "method");			
		}
		
				
	}

}
