#!/bin/sh

###--------input data----------------------------
dataset="$1" # [ant53, lucene37, rhino, aspectj]
which_method="$2" # [all, cv, slicing]
which_type="$3" # [class, method]
set_opt="$4"  # [union, intersection, none]
ir_type="$5"  # [r, s, baseline2, okapi] r: raw; s: structured

if [ $dataset == "ant53" ]
then
    echo "Running queries for dataset ... [$dataset]"
elif [ $dataset == "lucene37" ]
then
    echo "Running queries for dataset ... [$dataset]"
elif [ $dataset == "rhino" ]
then
    echo "Running queries for dataset ... [$dataset]"
elif [ $dataset == "aspectj" ]
then
    echo "Running queries for dataset ... [$dataset]"
else
    echo "[error] Wrong project name."
    exit -1
fi

if [ $which_method == "all" ]
then
    echo "Running IR-based ONLY."
elif [ $which_method == "cv" ]
then
    echo "Running IR-based with coverage."
    if [ $set_opt == "union" ]
    then
	echo "IR with Union Coverage"
    elif [ $set_opt == "intersection" ]
    then
	echo "IR with Intersection Coverage"
    else
	echo "[error] Put in the set operator, either union or intersection"
	exit
    fi    
elif [ $which_method == "slicing" ]
then
    echo "Running IR-based with slicing"
    if [ $set_opt == "union" ]
    then
	echo "IR with Union Slicing"
    elif [ $set_opt == "intersection" ]
    then
	echo "IR with Intersection Slicing"
    else
	echo "[error] Put in the set operator, either union or intersection"
	exit
    fi
else
    echo "[error] Wrong technique name."
    exit -1
fi

if [ $which_type == "class" ]
then
    echo "Running queries at class level"
elif [ $which_type == "method" ]
then
    echo "Ruuning quereis at method level"
else
    echo "[error] wrong level name: must be either: class or method"
fi


if [ $ir_type == "r" ]
then
    queries="queries"
    results="results"
    corpuses="corpuses"
elif [ $ir_type == "s" ]
then    
    queries="structured-queries"
    results="structured-results"
    corpuses="structured-corpuses"
elif [ $ir_type == "baseline2" ]
then
    queries="queries-baseline2"
    results="baseline2-results"
    corpuses="corpuses"
elif [ $ir_type == "okapi" ]
then
    queries="queries-okapi"
    results="okapi-results"
    corpuses="corpuses"
else
    echo "[error] incorrect ir type"
fi


echo "Running queries with the following settings:"
echo "[project] ... [$dataset]"
echo "[technique] ... [$which_method]"
echo "[level ] ... [$which_type]"
echo "[query type] ... [$ir_type]"


base="/Users/assist/dmz"
QUERIES_DIR="$base/ir-dataset/$queries/$dataset"


######## INPUT / OUTPUT FOLDERS #########################
CORPUS_DIR="$base/gen-data/ir/$which_type-ir/$dataset/$which_method/$corpuses"
RESULTS_DIR="$base/gen-data/ir/$which_type-ir/$dataset/$which_method/$results"

#CORPUS_DIR="$base/gen-data/ir_and_slicing/tmpdata/$dataset/index"
#RESULTS_DIR="$base/gen-data/ir_and_slicing/out/$dataset"

#CORPUS_DIR="$base/gen-data/ir/class-ir/aspectj/cv/union/corpuses"
#RESULTS_DIR="$base/gen-data/ir/class-ir/aspectj/cv/union/results"



if [ $which_method == "cv" ]
then
    echo ">>> You are running with set operation: $set_opt"
    CORPUS_DIR="$base/gen-data/ir/$which_type-ir/$dataset/$which_method/$set_opt/$corpuses"
    RESULTS_DIR="$base/gen-data/ir/$which_type-ir/$dataset/$which_method/$set_opt/$results"
fi


if [ ! -d "$RESULTS_DIR" ]
then
    mkdir "$RESULTS_DIR"
    mkdir "$RESULTS_DIR/post-fix"
    mkdir "$RESULTS_DIR/pre-fix"
else
    #rm -rfv $RESULTS_DIR/*
    rm -rfv $RESULTS_DIR/post-fix/*
    #    rm -rfv $RESULTS_DIR/*
    rm -rfv $RESULTS_DIR/pre-fix/*    
fi

QUERIES=$(ls $CORPUS_DIR)

for q in $QUERIES
do
    
    QUERY_FILE="$QUERIES_DIR/$q"
    
    VERSIONS=$(ls $CORPUS_DIR/$q)

    for i in $VERSIONS
    do
	temp=$(echo "$i" | grep "post-fix")
	
	if [ $? == 0 ]
    	then
	    indrirunquery $QUERY_FILE -index=$CORPUS_DIR/$q/$i -trecFormat=true -count=5000000 > "$RESULTS_DIR/post-fix/$q"
	    echo  "$q ->post-fix ... [ok]"
	else
	    indrirunquery $QUERY_FILE -index=$CORPUS_DIR/$q/$i -trecFormat=true -count=5000000 > "$RESULTS_DIR/pre-fix/$q"
	    echo  "$q ->pre-fix ... [ok]"
	fi	    
    done

    echo "$q .... [done]"

done

echo "Done !"

exit

### invocking ruby script to print out the final results ###
ground_truth_dir="$base/ir-dataset/ground_truth/$dataset"
#output_name="$dataset.$which_method.$which_type.$set_opt.$ir_type"
output_name="class.cv.aspectj"

#ruby f-compose-results.rb "$RESULTS_DIR/post-fix" "$output_name"  "$which_type" "$RESULTS_DIR/$output_name.post-fix.json" "$ground_truth_dir"
#ruby f-compose-results.rb "$RESULTS_DIR/pre-fix" "$output_name"  "$which_type" "$RESULTS_DIR/$output_name.pre-fix.json" "$ground_truth_dir"
#ruby f-results-json-to-text.rb "$RESULTS_DIR/$output_name.pre-fix.json" "$output_name" "$RESULTS_DIR/$output_name.pre-fix.txt"

#ruby f-compose-results.rb "$RESULTS_DIR" "$output_name"  "$which_type" "$RESULTS_DIR/$output_name.json" "$ground_truth_dir"
#ruby f-results-json-to-text.rb "$RESULTS_DIR/$output_name.json" "$output_name" "$RESULTS_DIR/$output_name.txt"
