#!bash/sh
project="$1"
spectrum="$2" # either coverage or slicing or method_coverage or method_slicing

if [ $project == "lucene37" ]
then
    rpath="lucene/src"
    echo "working on the project LUCENE"
elif [ $project == "ant53" ]
then
    rpath="src/main"
    echo "working on the project ANT"
elif [ $project == "rhino" ]
then
    rpath="src"
    echo "Working on the project RHINO"
elif [ $project == "aspectj" ]
then
    rpath="org.aspectj"
    echo "Working on the project ASPECTJ"
else
    echo "[error] incorrect project name"
fi

sir_jar_path="/Users/assist/dmz/tools/sIR/BLUiR.jar"
bug_dir="/Users/assist/dmz/ir-dataset/bugs-xml-repository/$project" #input of bug reports, must be existed

#code_dir="/Users/assist/dmz/ir-dataset/code/$project" # point to code of a project, including: pre-fix and post-fix
code_dir="/Users/assist/dmz/filter-code/$spectrum/$project" # point to code of a project, including: pre-fix and post-fix

if [ $spectrum == "coverage" ]
then
    output_dir="/Users/assist/dmz/gen-data/sIR_and_cv/out/$project" # store all the output results generated by the tool
    tmp_dir="/Users/assist/dmz/gen-data/sIR_and_cv/tmpdata/$project" # store all the intermediate data at each step generated by the tool
elif [ $spectrum == "slicing" ]
then    
    output_dir="/Users/assist/dmz/gen-data/sIR_and_slicing/out/$project" # store all the output results generated by the tool
    tmp_dir="/Users/assist/dmz/gen-data/sIR_and_slicing/tmpdata/$project" # store all the intermediate data at each step generated by the tool
elif [ $spectrum == "method_coverage" ]
then
    output_dir="/Users/assist/dmz/gen-data/sIR_method_and_cv/out/$project"
    tmp_dir="/Users/assist/dmz/gen-data/sIR_method_and_cv/tmpdata/$project"
elif [ $spectrum == "method_slicing" ]
then
    output_dir="/Users/assist/dmz/gen-data/sIR_method_and_slicing/out/$project"
    tmp_dir="/Users/assist/dmz/gen-data/sIR_method_and_slicing/tmpdata/$project"    
fi

corpusdir="/Users/assist/dmz/gen-data/ir/method-ir/$project/slicing/java"
          

if [ ! -d "$output_dir" ]
then
    mkdir -p "$output_dir"
else
    rm -rfv $output_dir/*
fi

if [ -d "$tmp_dir" ]
then
    rm -rfv $tmp_dir/query/*
    rm -rfv $tmp_dir/index/*
    rm -rfv $tmp_dir/docs/*    
else
    mkdir -p "$tmp_dir"
    mkdir "$tmp_dir/query"
    mkdir "$tmp_dir/index"
    mkdir "$tmp_dir/docs"
fi

bugs=$(ls $bug_dir) # list all bugs report in xml file (single bug in each file)

for b in $bugs # b is in .xml format
do
    id=$(echo $b | cut -f1 -d ".")  # get the id of a bug report (e.g., ANT-xxxx)

    if [ ! -d $corpusdir/$id ]
    then
	continue
    fi
    
    
    echo "Working on $id"
    
    java -jar "$sir_jar_path" -task=createquery -bugRepoLocation="$bug_dir/$b" -queryFilePath="$tmp_dir/query/$id"

#    java -jar "$sir_jar_path" -task=createdocs -codeLocation="$code_dir/$id" -docLocation="$tmp_dir/docs/$id"
    
    #id1=$(echo "$id" | cut -f2 -d "-")
    
    #java -jar "$sir_jar_path" -task=createdocs -codeLocation="$code_dir/$id1" -docLocation="$tmp_dir/docs/$id"
    
#    java -jar "$sir_jar_path" -task=index -indexLocation="$tmp_dir/index/$id" -docLocation="$tmp_dir/docs/$id"
    
    #    java -jar "$sir_jar_path" -task=retrieve -queryFilePath="$tmp_dir/query/$id" -indexLocation="$tmp_dir/index/$id" -resultPath="$output_dir/$id.txt" -topN=1000000

    java -jar "$sir_jar_path" -task=index -indexLocation="$tmp_dir/index/$id" -docLocation="$corpusdir/$id/post-fix"

    java -jar "$sir_jar_path" -task=retrieve -queryFilePath="$tmp_dir/query/$id" -indexLocation="$tmp_dir/index/$id" -resultPath="$output_dir/$id.txt" -topN=1000000

done

echo "Done!"
